!$Id: 
!
!=======================================================================
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ FLEXPART @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!=====|==1=========2=========3=========4=========5=========6=========7==
!
      subroutine gridcheck(oronew,error)
!**********************************************************************
!                                                                     * 
!             TRAJECTORY MODEL SUBROUTINE GRIDCHECK                   *
!             FLEXPART VERSION -> DO NOT USE IN FLEXTRA, PRAEPRO      *
!                                                                     *
!**********************************************************************
!                                                                     * 
!             AUTHOR:      G. WOTAWA                                  *
!             DATE:        1997-08-06                                 *
!             LAST UPDATE: 1997-10-10                                 *
!                                                                     * 
!             Update:      1999-02-08, global fields allowed, A. Stohl* 
!                                                                     * 
!**********************************************************************
!                                                                     *
! DESCRIPTION:                                                        *
!                                                                     *
! THIS SUBROUTINE DETERMINES THE GRID SPECIFICATIONS (LOWER LEFT      *
! LONGITUDE, LOWER LEFT LATITUDE, NUMBER OF GRID POINTS, GRID DIST-   *
! ANCE AND VERTICAL DISCRETIZATION OF THE ECMWF MODEL) FROM THE       *
! GRIB HEADER OF THE FIRST INPUT FILE. THE CONSISTANCY (NO CHANGES    *
! WITHIN ONE FLEXPART RUN) IS CHECKED IN THE ROUTINE "READWIND" AT    *
! ANY CALL.                                                           *
!                                                                     *
! OUTPUT       error .true.   - can not read grid specifications      *
!              error .false.  - normal                                *
!              oronew .true.  - Terrain heights given in grib files   *
!              oronew .false. - Terrain heights not specified in the  *
!                               grib files (old file standard)        *
!                                                                     * 
! XLON0                geographical longitude of lower left gridpoint *
! YLAT0                geographical latitude of lower left gridpoint  *
! NX                   number of grid points x-direction              *
! NY                   number of grid points y-direction              *
! DX                   grid distance x-direction                      *
! DY                   grid distance y-direction                      *
! NUVZ                 number of grid points for horizontal wind      *
!                      components in z direction                      *
! NWZ                  number of grid points for vertical wind        *
!                      component in z direction                       *
! sizesouth, sizenorth give the map scale (i.e. number of virtual grid*
!                      points of the polar stereographic grid):       *
!                      used to check the CFL criterion                *
! UVHEIGHT(1)-         heights of gridpoints where u and v are        *
! UVHEIGHT(NUVZ)       given                                          *
! WHEIGHT(1)-          heights of gridpoints where w is given         *
! WHEIGHT(NWZ)                                                        *
!                                                                     *
!**********************************************************************
!
      include 'includepar'
      include 'includecomB'

      integer i,ifn,ifield,j,k,iumax,iwmax,numskip
      real xaux1,xaux2,yaux1,yaux2,sizesouth,sizenorth,xauxa
      logical error,oronew

! VARIABLES AND ARRAYS NEEDED FOR GRIB DECODING

! dimension of isec2 at least (22+n), where n is the number of parallels or
! meridians in a quasi-regular (reduced) Gaussian or lat/long grid

! dimension of zsec2 at least (10+nn), where nn is the number of vertical
! coordinate parameters

      integer isec0(2),isec1(56),isec2(22+nxmax+nymax),isec3(2)
      integer isec4(64),inbuff(jpack),ilen,iswap,ierr,iword,lunit
      real zsec2(60+2*nuvzmax),zsec3(2),zsec4(jpunp)
      character*1 yoper,opt
      data yoper/'D'/


      error=.false.
      oronew=.false.
      iumax=0
      iwmax=0
!
      if(ideltas.gt.0) then
        ifn=1
      else
        ifn=numbwf
      endif
!
! OPENING OF DATA FILE (GRIB CODE)
!
5     call pbopen(lunit,path(3)(1:len(3))//wfname(ifn),'r',ierr)
      if(ierr.lt.0) goto 999

      ifield=0   
10    ifield=ifield+1   ! return point of the loop on fields
!
! GET NEXT FIELDS
!
      call pbgrib(lunit,inbuff,jpack,ilen,ierr)
      if(ierr.eq.-1) goto 30    ! EOF DETECTED
      if(ierr.lt.-1) goto 999   ! ERROR DETECTED
 
      ierr=1

! Check whether we are on a little endian or on a big endian computer
!********************************************************************

!      if (inbuff(1).eq.1112101447) then         ! little endian, swap bytes
      if(kind(1)==4) then   
        iswap=1+ilen/4
        call swap32(inbuff,iswap)
      else
        iswap=1+ilen/8
        call swap64(inbuff,iswap)
      endif
!      else if (inbuff(1).ne.1196575042) then    ! big endian
!        stop 'subroutine gridcheck: corrupt GRIB data'
!      endif

      call gribex(isec0,isec1,isec2,zsec2,isec3,zsec3,isec4, &
                  zsec4,jpunp,inbuff,jpack,iword,yoper,ierr)
      if (ierr.ne.0) goto 999   ! ERROR DETECTED
      
      if(ifield.eq.1) then
        nxfield=isec2(2)
        ny=isec2(3)
        xaux1=float(isec2(5))/1000.
        xaux2=float(isec2(8))/1000.
        print *,'gridcheck> xaux1, xaux2 ',xaux1,xaux2
        if (xaux1.ge. 179.999)  xaux1=xaux1-360.0
        if (xaux2.ge. 179.999)  xaux2=xaux2-360.0
        if (xaux1.lt.-180.001) xaux1=xaux1+360.0
        if (xaux2.lt.-180.001) xaux2=xaux2+360.0
        if (xaux2.lt.xaux1) xaux2=xaux2+360.
        yaux1=float(isec2(7))/1000.
        yaux2=float(isec2(4))/1000.
        xlon0=xaux1
        ylat0=yaux1
        dx=(xaux2-xaux1)/float(nxfield-1)
        dy=(yaux2-yaux1)/float(ny-1)
        dxconst=180./(dx*r_earth*pi)
        dyconst=180./(dy*r_earth*pi)
        nlev_ec=isec2(12)/2-1         !  Number of (u,v) ECMWF levels


! Check whether fields are global
! If they contain the poles, specify polar stereographic map
! projections using the stlmbr- and stcm2p-calls
!***********************************************************

        xauxa=abs(xaux2+dx-360.-xaux1)
        if (xauxa.lt.0.001) then
          nx=nxfield+1                 ! field is cyclic
          xglobal=.true.
        else
          nx=nxfield
          xglobal=.false.
        endif
        xauxa=abs(yaux1+90.)
        if (xglobal.and.xauxa.lt.0.001) then
          sglobal=.true.               ! field contains south pole
! Enhance the map scale by factor 3 (*2=6) compared to north-south
! map scale
          sizesouth=6.*(switchsouth+90.)/dy
          call stlmbr(southpolemap,-90.,0.)
          call stcm2p(southpolemap,0.,0.,switchsouth,0.,sizesouth, &
          sizesouth,switchsouth,180.)
          switchsouthg=(switchsouth-ylat0)/dy
        else
          sglobal=.false.
          switchsouthg=999999.
        endif
        xauxa=abs(yaux2-90.)
        if (xglobal.and.xauxa.lt.0.001) then
          nglobal=.true.               ! field contains north pole
! Enhance the map scale by factor 3 (*2=6) compared to north-south
! map scale
          sizenorth=6.*(90.-switchnorth)/dy
          call stlmbr(northpolemap,90.,0.)
          call stcm2p(northpolemap,0.,0.,switchnorth,0.,sizenorth, &
          sizenorth,switchnorth,180.)
          switchnorthg=(switchnorth-ylat0)/dy
        else
          nglobal=.false.
          switchnorthg=999999.
        endif
      endif
      
      select case (isec1(6))
      case (129)
        oronew=.true.
        k=isec1(8)  ! level number
        do j=0,ny-1
          do i=0,nxfield-1
            oro(i,j)=zsec4(nxfield*(ny-j-1)+i+1)/ga
          enddo
          if (xglobal) oro(nx-1,j)=oro(0,j)
        enddo
      case (131)                                                    
        iumax=max(iumax,nlev_ec-k+1) ! do we expect to get 
      case (135)
        iwmax=max(iwmax,nlev_ec-k+1) ! iumax or iwmax != nlev_ec
      case (172)
        do j=0,ny-1
          do i=0,nxfield-1
            lsm(i,j)=zsec4(nxfield*(ny-j-1)+i+1)
          enddo
          if (xglobal) lsm(nx-1,j)=lsm(0,j)
        enddo
      case (160)
      if(isec1(6).eq.160) then
        do j=0,ny-1
          do i=0,nxfield-1
            excessoro(i,j)=zsec4(nxfield*(ny-j-1)+i+1)
          enddo
          if (xglobal) excessoro(nx-1,j)=excessoro(0,j)
          enddo
      endif
      end select

      goto 10                      !! READ NEXT LEVEL OR PARAMETER
!
! CLOSING OF INPUT DATA FILE
!
30    call pbclose(lunit,ierr)     !! FINISHED READING / CLOSING GRIB FILE

      nuvz=iumax
      nwz =iwmax
      if(nuvz.eq.nlev_ec) nwz=nlev_ec+1  !  One more point at the top for the vertical velocity with w=0

      if (nx.gt.nxmax) then                         
      write(*,*) 'Trajectory  model error: Too many grid points'// &
                 ' in x direction.'
      write(*,*) 'Reduce resolution of wind fields.'
      error=.true.
      return
      endif

      if (ny.gt.nymax) then                         
      write(*,*) 'FLEXPART error: Too many grid points in y direction.'
      write(*,*) 'Reduce resolution of wind fields.'
      error=.true.
      return
      endif

      if (nuvz+1.gt.nuvzmax) then                         
      write(*,*) 'FLEXPART error: Too many u,v grid points in z '// &
        'direction.'
      write(*,*) 'Reduce resolution of wind fields.'
      error=.true.
      return
      endif

      if (nwz.gt.nwzmax) then                         
      write(*,*) 'FLEXPART error: Too many w grid points in z '// &
        'direction.'
      write(*,*) 'Reduce resolution of wind fields.'
      error=.true.
      return
      endif

! Output of grid info
!********************

      write(*,*)
      write(*,*)
      write(*,'(a,2i7)') '# of vertical levels in ECMWF data: ',  &
         nuvz+1,nwz
      write(*,*)
      write(*,'(a)') 'Mother domain:'
      write(*,'(a,f10.1,a1,f10.1,a,f10.1)') '  Longitude range: ', &
         xlon0,' to ',xlon0+(nx-1)*dx,'   Grid distance: ',dx
      write(*,'(a,f10.1,a1,f10.1,a,f10.1)') '  Latitude range: ',  &
         ylat0,' to ',ylat0+(ny-1)*dy,'   Grid distance: ',dy
      write(*,*)


! CALCULATE VERTICAL DISCRETIZATION OF ECMWF MODEL
! PARAMETER akm,bkm DESCRIBE THE HYBRID "ETA" COORDINATE SYSTEM

      numskip=nlev_ec-nuvz  ! number of ecmwf model layers not used
                            ! by trajectory model
      do  i=1,nwz
        j=numskip+i
        k=nlev_ec+1+numskip+i
        akm(nwz-i+1)=zsec2(10+j)
        bkm(nwz-i+1)=zsec2(10+k)
      enddo
!     do i=1,nwz
!        print*,'gridcheck> akm, bkm ',akm(i),bkm(i)
!      enddo

! Assign the 10 m winds to an artificial model level with akz=0 and bkz=1.0,
! i.e. ground level
!*****************************************************************************
 
      akz(1)=0.
      bkz(1)=1.0
      do 90 i=1,nuvz
         akz(i+1)=0.5*(akm(i+1)+akm(i))
         bkz(i+1)=0.5*(bkm(i+1)+bkm(i))
90    enddo
      nuvz=nuvz+1 		! one more point at the bottom with 10m velocities
      print *,'gridcheck> nwz, nuvz ',nwz,nuvz

!  In order to avoid log(0), we pull down the upper boundary from
!  zero pressure to the top layer for (u,v)
!  vertical velocity at this level is defined in readwinds.f

      akm(nwz) = akz(nuvz) ; bkm(nwz) = bkz(nuvz)

      return

999   write(*,*)  
      write(*,*) ' ###########################################'// &
                 '###### '
      write(*,*) '       TRAJECTORY MODEL SUBROUTINE GRIDCHECK:'
      write(*,*) ' CAN NOT OPEN INPUT DATA FILE '//wfname(ifn)
      write(*,*) ' ###########################################'// &
                 '###### '
      write(*,*)
      write(*,'(a)') '!!! PLEASE INSERT A NEW CD-ROM AND   !!!'
      write(*,'(a)') '!!! PRESS ANY KEY TO CONTINUE...     !!!'
      write(*,'(a)') '!!! ...OR TERMINATE FLEXTRA PRESSING !!!'
      write(*,'(a)') '!!! THE "X" KEY...                   !!!'
      write(*,*)
      read(*,'(a)') opt
      if(opt.eq.'X') then
        error=.true.
      else
        goto 5
      endif

      return
      end
!
!=====|==1=========2=========3=========4=========5=========6=========7==
!
!$Log: 
