!$Id:
!
!=======================================================================
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ FLEXPART @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!=====|==1=========2=========3=========4=========5=========6=========7==
!
 subroutine readreleasesB2(error)

!*******************************************************************************
! 
!     This routine reads the release point specifications for the current 
!     model run. Several release points can be used at the same time.
!     meant to replace readreleasesB, and to have original readreleases in place 
! 
!     Author: B. Legras
! 
!     1 June 2002
! 
!*******************************************************************************
!                                                                              *
! Variables:                                                                   *
! decay               decay constant of species                                *
! dquer [um]          mean particle diameters                                  *
! dsigma              e.g. dsigma=10 or dsigma=0.1 means that 68% of the mass  *
!                     are between 0.1*dquer and 10*dquer                       *
! ireleasestart, ireleaseend [s] starting time and ending time of each release *
! npart               number of particles to be released                       *
! nspec               number of species to be released                         *
! density [kg/m3]     density of the particles                                 *
! rm [s/m]            Mesophyll resistance                                     *
! species             name of species                                          *
! xmass               total mass of each species                               *
! xpoint1,ypoint1     geograf. coordinates of lower left corner of release area*
! xpoint2,ypoint2     geograf. coordinates of upper right corner of release are*
! weta, wetb          parameters to determine the wet scavenging coefficient   *
! zpoint1,zpoint2     height range, over which release takes place             *
!                                                                              *
!*******************************************************************************

 use commons
 use lyapunov, only: activ_Lyapunov, height_scale, sheet_slope, delta_ver, &
    delta_hor, tau_orthog
 use isentrop_h
 use mass_iso
 implicit none

 integer numpartmax,i,j,id1,it1,id2,it2
 integer launch_date,launch_time,nb_level
 real lat_station,long_station,p_min,p_max
 real long_min,long_max,lat_min,lat_max,press_l,theta_l,mesh_size
 double precision jul1,jul2,juldate
 character(len=50):: line
 character(len=20):: name_station
 logical, intent(out):: error

 namelist /O3SONDE/    &
    name_station,      & ! name of launching station
    long_station,      & ! longitude of launching station (deg)
    lat_station,       & ! latitude of launching station  (deg)
    p_min, p_max,      & ! lower and upper p level (Pa above ground)
    nb_level, n_sample,& ! number of levels to define, sammple size per level
    launch_date,       & ! launch date (yyyymmdd)
    launch_time          ! launch time (hhmmss)
 namelist /TTL/         & 
    long_min,           & ! minimum longitude
    long_max,           & ! maximum longitude
    lat_min ,           & ! minimum latitude
    lat_max ,           & ! maximum latitude
    press_l,            & ! launch pressure
    theta_l,            & ! launch theta 
    mesh_size,          & ! size of the mesh in degree
    launch_date,        & ! launch date (yyyymmdd)
    launch_time,        & ! launch time (hhmmss)
    n_sample,           & ! sample size per point
    TTLactiv              ! activation of humidity saturation calculations
 namelist /AGE/         & 
    long_min,           & ! minimum longitude
    long_max,           & ! maximum longitude
    lat_min ,           & ! minimum latitude
    lat_max ,           & ! maximum latitude
    press_l,            & ! launch pressure
    theta_l,            & ! launch theta 
    mesh_size,          & ! size of the mesh in degree
    launch_date,        & ! launch date (yyyymmdd)
    launch_time,        & ! launch time (hhmmss)
    n_sample,           & ! sample size per point
    uniform_spread,     & !
    make_curtain,       &
    make_layer,         &
    curtain_type
 namelist /MOZAIC/      &
    MOZAIC_dir,         & ! MOZAIC directory
    MOZAIC_filename,    & ! name of the file containing flight data
    start_index,        & ! beginning of the analysed segment
    end_index,          & ! end of the analysed segment
    index_type,         & ! index type, 'rank' or 'time'
    n_sample,           & ! number of particles in each sample
    instant_release,    & ! if .true., release at observed time with partial step
    interp_release,     & ! if .true., interpolated release
    n_loc                 ! number of points (if interp_release=.true.)
 namelist /ER2/         &
    ER2_dir,            & ! ER2 directory
    ER2_day,            & ! flight day	
    start_ER2time,      & ! start time of the analysed segment
    end_ER2time,        & ! end time of the analysed segment
    n_sample,           & ! number of particles in each sample
    instant_release,    & ! if .true., release at observed time with partial step
    interp_release,     & ! if .true., interpolated release
    n_loc,              & ! number of points (if interp_release=.true.)
    campaign              ! name of the campaign

! Initialize logical variables
!-----------------------------
 delayed_initialization=.false.
 press2theta=.false.
   
! Open the releases file and read user options
!---------------------------------------------

 error=.false.
 open(unitreleases,file=path(1)(1:len(1))//'RELEASES',status='old',err=999)

! Check the format of the RELEASES file and call old readcommand
! if new heading with RELEASES in the first line is not detected
!--------------------------------------------------------------

 read (unitreleases,'(a)') line
 if (index(line,'RELEASES') == 0 ) then
   close(unitreleases)
   releases_old=.true.
   call readreleasesB(error)
   return
 endif

 select case (release_plan)

 ! Launch particles at a given time to match an
 ! CO2 simulation
! case ('CO2')
!    nspec=1
!    numpoint=1
!read(unitreleases,CO2)
!    zpoint1(1) = p_min ; zpoint2(1) = p_max
!    jul1=juldate(launch_date,launch_time)
!    numlevel(1) = nb_level
!    nsample(1)  = n_sample
!    long_min = xlon0
!    long_max = xlon0 + (nx-2)*dx ! global assumed
!    lat_min = -90.
!    lat_max =  90.
!    xpoint1(1)=long_min; xpoint2(1)=long_max
!    ypoint1(1)=lat_min;  ypoint2(1)=lat_max
!    mesh_size_lat=mesh_size
!    mesh_size_long=mesh_size
!    TTLactiv=.false.
!    AccurateTemp=.true.
!    uniform_spread=.true.
!    savfull=.false.
!    make_curtain=.false.
!    curtain_type=1
!    make_layer=.true.

!ozone souding
 case ('o3sonde')
    nspec=1
    numpoint=1
    link(1)=1
read(unitreleases,O3SONDE)
    xpoint1(1) = long_station ; xpoint2(1)=xpoint1(1)
    ypoint1(1) = lat_station  ; ypoint2(1)=ypoint1(1)
    zpoint1(1) = p_min ; zpoint2(1) = p_max
    jul1=juldate(launch_date,launch_time)
    ireleasestart(1) = int((jul1-bdate)*86400.)
    ireleaseend (1) = ireleasestart(1)
    numlevel(1) = nb_level
    nsample(1)  = n_sample
    npart(1) = nb_level * n_sample
    if(npart(1) > maxpart) go to 996
    error=(check_input_date()==1).or.error
    print *,'readreleases> O3SONDE'
    print *,'readreleases> ',launch_date,launch_time
    print *,'readreleases> ',xpoint1(1),ypoint1(1)
    print *,'readreleases> ',zpoint1(1),zpoint2(1)
    print *,'readreleases> ',numlevel(1),nsample(1)
 ! Releases particles in the TTL
 ! Type a: particles are released within a lat-long box
 ! at a fixed pressure level
 case('TTL')
    nspec=1
    numpoint=1
    n_sample=1
    long_min = xlon0
    long_max = xlon0 + (nx-2)*dx ! global assumed
    press_l=0.
    theta_l=0.
    n_loc=0
    TTLactiv=.true.
    uniform_spread=.false.
    AccurateTemp=.true.
    savfull=.true.
    read(unitreleases,TTL)
    xpoint1(1)=long_min; xpoint2(1)=long_max
    ypoint1(1)=lat_min;  ypoint2(1)=lat_max
    if(press_l > 0.) then
      if(diabatic_w .or. isentropic_motion) then
        delayed_initialization=.true.
        press2theta=.true.
      endif
      zpoint1(1)=-log(press_l/p0)
    else if (theta_l > 0.) then
      if(z_motion) then
        delayed_initialization=.true.
        theta2press=.true.
      endif
      zpoint1(1)=theta_l
    else
      go to 998
    endif
    nsample(1)=n_sample
    ! calculation of n_loc delayed to fixlayerpart
    jul1=juldate(launch_date,launch_time)
    ireleasestart(1) = int((jul1-bdate)*86400.)
    ireleaseend (1) = ireleasestart(1)
    error=(check_input_date()==1).or.error
    mesh_size_lat=mesh_size
    mesh_size_long=mesh_size
    print *,'readreleases> TTL'
    print *,'readreleases> date ',launch_date,launch_time
    print *,'readreleases> min long lat (deg) ',xpoint1(1),ypoint1(1)
    print *,'readreleases> max long lat (deg) ',xpoint2(1),ypoint2(1)
    print *,'readreleases> pressure (Pa) ',press_l
    print *,'readreleases> theta (K) ',theta_l
    print *,'readreleases> mesh_size (deg) ',mesh_size
    print *,'readreleases> n_sample ',nsample(1)
! Age of air calculation (akin to TTL)
 case('AGE')
    print *,'readreleases> AGE OF AIR'
    nspec=1
    numpoint=1
    n_sample=1
    long_min = xlon0
    long_max = xlon0 + (nx-2)*dx ! global assumed
    lat_min = -90.
    lat_max =  90.
    press_l=0.
    theta_l=0.
    n_loc=0
    TTLactiv=.false.
    AccurateTemp=.true.
    uniform_spread=.true.
    savfull=.false.
    make_curtain=.false.
    curtain_type=1
    make_layer=.true.
    print *,'passed ctrl 0'
    read(unitreleases,AGE)
    xpoint1(1)=long_min; xpoint2(1)=long_max
    ypoint1(1)=lat_min;  ypoint2(1)=lat_max
    nsample(1)=n_sample
    print *,'passed ctrl 1'
    if(press_l > 0.) then
      if(iso_mass .or. diabatic_w .or. isentropic_motion) then
        delayed_initialization=.true.
        press2theta=.true.
      endif
      zpoint1(1)=-log(press_l/p0)
    else if (theta_l > 0.) then
      if(z_motion) then
        delayed_initialization=.true.
        theta2press=.true.
      endif
      zpoint1(1)=theta_l
    else
      go to 998
    endif
    nsample(1)=n_sample
    ! calculation of n_loc delayed to fixlayerpart
    jul1=juldate(launch_date,launch_time)
    ireleasestart(1) = int((jul1-bdate)*86400.)
    ireleaseend (1) = ireleasestart(1)
    error=(check_input_date()==1).or.error
    mesh_size_lat=mesh_size
    mesh_size_long=mesh_size
    if (make_curtain) then
      select case (curtain_type)
      case (1)  
        allocate(lat_list(15))
        lat_list = (/ -70.,-60.,-50.,-40.,-30.,-20.,-10.,0.,10.,20.,30.,40.,50.,60.,70. /)
        lev1 = 39.
        lev2 = 57.
        inclev= 1.
        upperthetalevel=2485.30
      case (2)
        print *,'readreleasesB2> WARNING: specific option will not work for all cases'
        make_layer=.false.
        allocate(lat_list(31))
        do i=1,31
          lat_list(i)=2.*i-32.
        enddo
        lev1 = 31.
        lev2 = 40.
        inclev = 0.5
        upperthetalevel=2485.30
        numpoint = 8
        xpoint1(1:8)=long_min
        xpoint2(1:8)=long_max
        nsample(1:8)=n_sample
        zpoint1(2:8)=zpoint1(1) ! just to avoid WARNING in fixlayerpart
        launch_time=120000
        ireleasestart(1) = int((juldate(20001231,launch_time)-bdate)*86400.)
        ireleasestart(2) = int((juldate(20001221,launch_time)-bdate)*86400.)
        ireleasestart(3) = int((juldate(20001211,launch_time)-bdate)*86400.)
        ireleasestart(4) = int((juldate(20001201,launch_time)-bdate)*86400.)
        ireleasestart(5) = int((juldate(20000731,launch_time)-bdate)*86400.)
        ireleasestart(6) = int((juldate(20000721,launch_time)-bdate)*86400.)
        ireleasestart(7) = int((juldate(20000711,launch_time)-bdate)*86400.)
        ireleasestart(8) = int((juldate(20000701,launch_time)-bdate)*86400.)
      end select        
    endif 
    print *,'readreleases> AGE OF AIR'
    print *,'readreleases> date ',launch_date,launch_time
    print *,'make_curtain ',make_curtain
    print *,'curtain_type ',curtain_type
    print *,'lev1 lev2 inclev ',lev1,lev2,inclev
    print *,'lat_list',lat_list(1),lat_list(size(lat_list)), &
             lat_list(2)-lat_list(1)
    print *,'readreleases> min long lat (deg) ',xpoint1(1),ypoint1(1)
    print *,'readreleases> max long lat (deg) ',xpoint2(1),ypoint2(1)
    if(press_l>0.) print *,'readreleases> pressure (Pa) ',press_l
    if(theta_l>0.) print *,'readreleases> potential temperature (K)) ',theta_l

 ! Releases particles along a MOZAIC flight
 case('MOZAIC')
    MOZAIC_dir='/net/parker/d502/legras/MOZAIC/'
    index_type='time'
    n_sample=1
    instant_release=.false.
    interp_release =.false.
    numpart=0
    n_loc=0
    read(unitreleases,MOZAIC)
    nsample(1) = n_sample
    if(numpart > 0) n_loc=numpart ! to keep valid old RELEASE files
    numlevel(1) = n_loc           ! to perform correct output of n_loc
    print *,'readreleases> MOZAIC'
    print *,'readreleases> ',MOZAIC_filename
    print *,'readreleases> ',index_type
    print *,'readreleases> ',start_index,end_index
    print *,'readreleases> n_sample ',n_sample
    print *,'readreleases> n_loc    ',n_loc
    print *,'readreleases> instant_release ',instant_release
    print *,'readreleases> interp_release  ',interp_release
 ! Releases particles along an ER2 flight assuming
 ! standardized NASA-Ames format of meteorological files
 case('ER2')
    campaign='SOLVE'
    ER2_dir='/user/legras/h4/NASA/SOLVE/er2'
    n_sample=1
    instant_release=.false.
    interp_release =.true.
    AccurateTemp=.true.
    numpart=0
    n_loc=0
    read(unitreleases,ER2)
    nsample(1) = n_sample
    if(numpart > 0) n_loc=numpart ! to keep valid old RELEASE files valid
    numlevel(1) = n_loc           ! to perform correct output of n_loc
    print *,'readreleases> ER2'
    print *,'readreleases> ',campaign
    print *,'readreleases> ',ER2_day
    print *,'readreleases> ',start_ER2time,end_ER2time
    print *,'readreleases> n_sample ',n_sample
    print *,'readreleases> n_loc    ',n_loc
    print *,'readreleases> instant_release ',instant_release
    print *,'readreleases> interp_release  ',interp_release
 case('ER2lyapou')
    activ_Lyapunov=.true.
    height_scale = 5000.
    sheet_slope = 250.
    delta_hor = 2500.
    tau_orthog = -43200
    campaign='SOLVE'
    ER2_dir='/user/legras/h4/NASA/SOLVE/er2'
    n_sample=1
    instant_release=.false.
    interp_release =.true.
    numpart=0
    n_loc=0
    read(unitreleases,ER2)
    call check_sample()
    nsample(1) = n_sample
    numlevel(1) = n_loc           ! to perform correct output of n_loc
    delta_ver = delta_hor / sheet_slope
    print *,'readreleases> ER2lyapou'
    print *,'readreleases> ',ER2_day
    print *,'readreleases> ',start_ER2time,end_ER2time
    print *,'readreleases> n_sample ',n_sample
    print *,'readreleases> n_loc    ',n_loc
    print *,'readreleases> instant_release ',instant_release
    print *,'readreleases> interp_release  ',interp_release 
    
 case default
   error=.true.
   write(*,*)  
   write(*,*) '#####################################################'
   write(*,*) '   FLEXPART MODEL SUBROUTINE READRELEASES2B: '
   write(*,*)
   write(*,*) 'FATAL ERROR - RELEASES PLAN NON AVAILABLE'
   write(*,*) '#####################################################'
   return
 end  select
 close(unitreleases) 

 return

999   error=.true.
      write(*,*)  
      write(*,*) '#####################################################'
      write(*,*) '   FLEXPART MODEL SUBROUTINE READRELEASES: '
      write(*,*)
      write(*,*) 'FATAL ERROR - FILE CONTAINING PARTICLE RELEASE POINTS'
      write(*,*) 'POINTS IS NOT AVAILABLE OR YOU ARE NOT'
      write(*,*) 'PERMITTED FOR ANY ACCESS'
      write(*,*) '#####################################################'
      return

996   error=.true.
      write(*,*) '#####################################################'
      write(*,*) '#### FLEXPART MODEL SUBROUTINE READRELEASES:     ####'
      write(*,*) '####                                             ####'
      write(*,*) '#### ERROR - TOTAL NUMBER OF PARTICLES SPECIFIED ####'
      write(*,*) '#### IN FILE "RELEASES" EXCEEDS THE MAXIMUM      ####'
      write(*,*) '#### ALLOWED NUMBER. REDUCE EITHER NUMBER OF     ####'
      write(*,*) '#### PARTICLES PER RELEASE POINT OR REDUCE NUMBER####'
      write(*,*) '#### OF RELEASE POINTS.                          ####'
      write(*,*) '#####################################################'
      return

998   error=.true.
      write(*,*) '#####################################################'
      write(*,*) '#### FLEXPART MODEL SUBROUTINE READRELEASES:     ####'
      write(*,*) '####                                             ####'
      write(*,*) '#### ERROR - MUST PROVIDE A VALUE FOR A PRESSURE ####'
      write(*,*) '#### OR THETA LEVEL ON WHICH TRACER ARE          ####'
      write(*,*) '#### INITIALIZED                                 ####'
      write(*,*) '#####################################################'
      return


 contains
      
      function check_input_date ()
      integer :: check_input_date
      check_input_date = 0
      if (ldirect.eq.1) then
        if ((jul1.lt.bdate).or.(jul1.gt.edate)) then
          write(*,*) 'FLEXPART MODEL ERROR'
          write(*,*) 'Release before simulation begins or '
          write(*,*) 'after simulation stops.'
          write(*,*) 'Make files COMMAND and RELEASES consistent.'
          check_input_date=1
          return
        endif        
      else if (ldirect.eq.-1) then
        if ((jul1.lt.edate).or.(jul1.gt.bdate)) then
          write(*,*) 'FLEXPART MODEL ERROR'
          write(*,*) 'Release after simulation begins or '
          write(*,*) 'before simulation stops.'
          write(*,*) 'Make files COMMAND and RELEASES consistent.'
          check_input_date=1 
          return
        endif
      endif
      return
      end function check_input_date
      
      subroutine check_sample ()
      if (n_sample > 1) then
        write(*,*) 'FLEXPART WARNING'
	write(*,*) 'n_sample forced to 1'
	n_sample = 1
      endif
      return
      end subroutine check_sample

 end subroutine readreleasesB2
!
!=====|==1=========2=========3=========4=========5=========6=========7==
!
!$Log:
