!$Id: 
!
!=======================================================================
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ FLEXPART @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!=====|==1=========2=========3=========4=========5=========6=========7==
!
      subroutine advanceB(jp,itime,dt,nstop,x,y,z,tint)
            
!*******************************************************************************
!                                                                              *
!  Calculation of trajectories utilizing a zero-acceleration scheme.           *
!  The time step is determined by the Courant-Friedrichs-Lewy (CFL) criterion. *
!  This means that the time step must be so small that the displacement within *
!  this time step is smaller than 1 grid distance. Additionally, a temporal    *
!  CFL criterion is introduced: the time step must be smaller than the time    *
!  interval of the wind fields used for interpolation.                         *
!  For random walk simulations, these are the only time step criteria.         *
!  For the other options, the time step is also limited by the Lagrangian time *
!  scale.                                                                      *
!                                                                              *
!     Author: A. Stohl                                                         *
!                                                                              *
!     16 December 1997                                                         *
!     
!     Changes: B. Legras, April 2002
!            boundary layer calculation cancelled
!              simple advection with vertical noise
!              B. Legras, September 2007
!              option for horizontal diffusion
!              B. Legras, September 2012
!              tests for pcut and thetacut
!                                                                              *
!*******************************************************************************
!                                                                              *
! Variables:                                                                   *
! h [m]              Mixing height                                             *
! ladvance [s]       Total integration time period                             *
! ngrid              index which grid is to be used                            *
! rannumb(maxrand)   normally distributed random variables                     *
! x,y,z              Next time step's spatial position of trajectory           *
!                                                                              *
!                                                                              *
!*******************************************************************************

      use commons
      use isentrop_m
      use ecmwf_diab
      use ecmwf_inct
      use mass_iso

      implicit none

      integer, parameter :: nss=50
      real, parameter :: pih=pi/180.,eps=1.e-4
      
      integer, intent(in):: itime, jp
      integer, intent(out):: nstop
      real, intent(in) :: dt
      real, intent(inout):: x,y,z
      real, intent(out):: tint

      integer idummy,i,j,k,nrand,loop
      integer memindnext,ixnext,jynext
      integer ix,jy,ixp,jyp,ngrid
      real h,hmixn,cgszll,ran3,wdiff
      real xlon,ylat,xpol,ypol,gridsize
      real ux,vy,cosfact,xtn,ytn,psaver,theta_inf,theta_sup
      real u,v,w,dxsave,dysave,deltap,rhog,z_factor,rand_angle
      real pint
      logical partial_step
      real, dimension(nss) :: harvest
      save idummy

      data idummy/-7/

      dxsave=0.           ! reset position displacements
      dysave=0.           ! due to mean wind
      nstop=0

!      nrand=int(ran3(idummy)*float(maxrand-1))+1

! Determine whether lat/long grid or polarstereographic projection
! is to be used
! Furthermore, determine which nesting level to be used
!*****************************************************************

      if (nglobal.and.(y.gt.switchnorthg)) then
        ngrid=-1
      else if (sglobal.and.(y.lt.switchsouthg)) then
        ngrid=-2
      else
        ngrid=0
      endif

!***************************
! Interpolate necessary data
!***************************

!      if (abs(itime-memtime(1)).lt.abs(itime-memtime(2))) then
!        memindnext=1
!      else
!        memindnext=2
!      endif

! Determines grid coordinate
     
!      ix=int(x)      ; jy=int(y)
!      ixp=ix+1       ; jyp=jy+1

! Compute maximum mixing height around particle position
!*******************************************************

!       h=0.

!       do k=1,2
!         do j=jy,jyp
!           do i=ix,ixp
!             if (hmix(i,j,1,memind(k)).gt.h) h=hmix(i,j,1,memind(k))
!           enddo
!         enddo
!       enddo


!*************************************************************
! Determines (roughly) if particle is in the PBL
! then do nothing for the moment
!*************************************************************

!      deltap = p0*exp(-z)-ps(ix,jy,1,2)
!      rhog   = ga*p0*exp(-z)/(r_air*tth(ix,jy,1,2))
!      if (deltap.lt.h*rhog) then
!        continue
!     endif

!**********************************************************
! For all particles that are outside the PBL, make a single
! time step. Only horizontal turbulent disturbances are
! calculated. Vertical disturbances are reset.
!**********************************************************
! TEST

! Interpolate the wind
!*********************

      if(isentropic_motion) then
        call interpol_wind_theta (itime,x,y, z, & 
           u,v, ngrid, theta_inf, theta_sup, z_factor,tint)
           if(debug_out) print *,'advanceB (u,v)> ',u,v             ! test
      elseif (diabatic_w .and. ecmwf_diabatic_w ) then
        if (ecmwf_inct_w) then
          call interpol_wind_theta_inct (jp,itime,x,y, z, & 
            u,v,w, ngrid, theta_inf, theta_sup, z_factor,tint,nstop)
            if(debug_out) print *,'advanceB (u,v,w)> ',u,v,w*86400  ! test
        else
          call interpol_wind_theta_diab (jp,itime,x,y, z, & 
            u,v,w, ngrid, theta_inf, theta_sup, z_factor,tint,nstop)
            if(debug_out) print *,'advanceB (u,v,w)> ',u,v,w*86400  ! test
        endif
      elseif (z_motion) then
        call interpol_windB(jp,itime,x,y,z,u,v,w,ngrid, &
           psaver,z_factor,tint)
           if(debug_out) print *,'advanceB (u,v,w)> ',u,v,w        ! test
      elseif (iso_mass) then
        call interpol_wind_iso(jp,itime, x, y, z, & 
           u, v, w, ngrid, z_factor, tint, nstop)
           if(debug_out) print *,'advanceB (u,v,w)> ',u,v,w*86400  ! test
      endif
      !if (z_motion) print *,z,tint,tint*exp(kappa*z)
      !if (diabatic_w .or. isentropic_motion) &
      !    print*,log(z/tint)/kappa,tint,z

! Compute everything for above the PBL

! TEST
!      if(jp==27385) print *, 'uvw  ',u,v,w

! Calculate position at time step itime+lsynctime
!************************************************
      dxsave=u*dt   ! unsigned displacement
      dysave=v*dt   ! ...
      if(z_motion) z=z-w*dt*float(ldirect)/(p0*exp(-z))  ! signed displacement
      if(diabatic_w .and. ecmwf_diabatic_w) z=z+w*dt*float(ldirect)
      if(iso_mass .and. mass_diabat)  z=z+w*dt*float(ldirect)

!****************************************************************
! Add random vertical component to the velocity
!****************************************************************
       
      if((nsample(1) > 1).and.(.not.switch_diff_off)) then
        call random_number(harvest)
        wdiff=2*sum(harvest) - nss
        wdiff = wdiff*sqrt(6*diffus/(nss*dt))
        if(verdiff) z=z+wdiff*dt*float(ldirect)*z_factor
        if(hordiff) then
          call random_number(rand_angle)
          rand_angle=2*pi*rand_angle
          dxsave = dxsave + wdiff*dt*cos(rand_angle)
          dysave = dysave + wdiff*dt*sin(rand_angle)
        endif
      endif    
      
!****************************************************************
! Projects onto the frontier if going above
!****************************************************************

      if (z_motion) then 
         z=min(z,zmax)
         if (z.lt.log(p0/psaver)) z=2*log(p0/psaver)-z ! bouncing
      endif
      if (diabatic_w.and.theta_bounds) &
         z=max(min(z,theta_sup),theta_inf)
      if (mass_diabat.and.theta_bounds) &
         z=max(min(z,ThetaLev(NbTheta)),ThetaLev(1))

!*************************************************************
! Transform along and cross wind components to xy coordinates,
! add them to u and v, transform u,v to grid units/second
! and calculate new position
!*************************************************************

      if (ngrid.ge.0) then
        cosfact=dxconst/cos((y*dy+ylat0)*pih)
        x=x+dxsave*cosfact*float(ldirect)
        y=y+dysave*dyconst*float(ldirect)
      else if (ngrid.eq.-1) then      ! around north pole
        xlon=xlon0+x*dx
        ylat=ylat0+y*dy
        call cll2xy(northpolemap,ylat,xlon,xpol,ypol)
        gridsize=1000.*cgszll(northpolemap,ylat,xlon)
        dxsave=dxsave/gridsize
        dysave=dysave/gridsize
        xpol=xpol+dxsave*float(ldirect)
        ypol=ypol+dysave*float(ldirect)
        call cxy2ll(northpolemap,xpol,ypol,ylat,xlon)
        x=(xlon-xlon0)/dx
        y=(ylat-ylat0)/dy
      else if (ngrid.eq.-2) then    ! around south pole
        xlon=xlon0+x*dx
        ylat=ylat0+y*dy
        call cll2xy(southpolemap,ylat,xlon,xpol,ypol)
        gridsize=1000.*cgszll(southpolemap,ylat,xlon)
        dxsave=dxsave/gridsize
        dysave=dysave/gridsize
        xpol=xpol+dxsave*float(ldirect)
        ypol=ypol+dysave*float(ldirect)
        call cxy2ll(southpolemap,xpol,ypol,ylat,xlon)
        x=(xlon-xlon0)/dx
        y=(ylat-ylat0)/dy
      endif


! If global data are available, use cyclic boundary condition
!************************************************************
     
      if (xglobal) x = modulo(x,float(nx-1))

!*****************************************************************
! Check position: If trajectory outside model domain, terminate it
!*****************************************************************

!     nstop must be less or equal to 7 or change bound in timemanager

      if ((x.lt.0.).or.(x.ge.nearest(float(nx-1),1.)).or.(y.lt.0.).or. &
       (y.ge.float(ny-1)).or.(z_motion.and.(z>zmax))) then   !trajectory terminated
        nstop=3
        print *, 'stop parcel:trajectory terminated ',jp, ngrid
        if (ngrid.ge.0) then
          write(*,'(9G14.5)') x,y,z,dxsave,dysave,w,u,v,dt
!         print *,y-dysave*dyconst*float(ldirect),switchnorthg
        else
          write(*,'(6g14.5)') xlon,ylat,dxsave,dysave,xpol,ypol
        endif
      ! write (*,'(a,3g12.3)')'stopped trajectory ',x,y,z
      endif

      if (AGEFactiv) then
        if (z_motion) then
          if (p0*exp(-z) > pcut) then
            nstop=4
          endif
        else
          if (p0*(tint/z)**(1/kappa) > pcut) then
            nstop=4
          else if (z > thetacut) then
            nstop=5
          endif
        endif
      endif

      return
      end
!
!=====|==1=========2=========3=========4=========5=========6=========7==
!
!$Log: 
!
!
