!$Id:
!
!=======================================================================
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ FLEXPART @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!=====|==1=========2=========3=========4=========5=========6=========7==
!
  subroutine fixER2SOLVEinterp(error)
!*******************************************************************************
!                                                                              *
!     This routine fixes the release times and release locations of all        *
!     particles.           						       *
!     Interpolate selected part of the flight path to n_loc points regularly
!     distributed in time. Put n_sample particles at each point.
!
!     Changes: B. Legras, Nov. 2002
!              version for distribution along an aircraft transect 
!              with temporal interpolation
!                                                                              *
!*******************************************************************************
!                                                                              *
! Variables:                                                                   *
! ideltas [s]          simulation period                                       *
!                                                                              *
!*******************************************************************************

 use commons
 use lyapunov, only: numpart_np, activ_Lyapunov
 use isentrop_m, only: isentropic_motion
 use ecmwf_diab

 implicit none
       
 real, allocatable, dimension(:) :: timeO3, UTGPS, GMTMM,  &
      interp_time, &
      interp_xtra1, interp_ytra1, interp_ztra1, &
      F_z, F_lat, F_long
 
 integer, allocatable, dimension(:) :: O3, &
      DaltGPS, DlatGPS, DLongGPS, QGPS, TedrGPS, ReynGPS, &
      PStaMM, TstaMM, ThtaMM, UMM, VMM, WMM
 
 logical, intent(out):: error
 logical do_rel,stop_loop
 integer lhead, FDate, YYYY, MM, DD
 integer i,j,j1,nobs,nO3obs,nGPSobs,nMMobs
 real diff_time 
 double precision juldate
 
 error=.false.
 print * ,trim(ER2_dir)//'O3'//trim(ER2_day)//'.ER2'
 open(unitER2_O3,file=trim(ER2_dir)//'O3'//trim(ER2_day)//'.ER2', &
      status='OLD',err=996,FORM='FORMATTED')
 print * ,trim(ER2_dir)//'MG'//trim(ER2_day)//'.ER2'
 open(unitER2_MG,file=trim(ER2_dir)//'MG'//trim(ER2_day)//'.ER2', &
      status='OLD',err=996,FORM='FORMATTED')
 print * ,trim(ER2_dir)//'MM'//trim(ER2_day)//'.ER2'
 open(unitER2_MM,file=trim(ER2_dir)//'MM'//trim(ER2_day)//'.ER2', &
      status='OLD',err=996,FORM='FORMATTED')
 read(unitER2_O3,*) lhead
 do i=2,lhead
   read(unitER2_O3,*)
 enddo   
 read(unitER2_MG,*) lhead
 do i=2,lhead
   read(unitER2_MG,*)
 enddo
 read(unitER2_MM,*) lhead
 do i=2,6
   read(unitER2_MM,*)
 enddo
 read(unitER2_MM,*) YYYY, MM, DD
 do i=8,lhead
   read(unitER2_MM,*)
 enddo 
 FDate=10000*YYYY + 100*MM + DD
 diff_time = end_ER2time - start_ER2time
 nobs=int(1.05*diff_time) ! rough estimate for less than 1Hz signal
 allocate (timeO3(nobs),O3(nobs),UTGPS(nobs),GMTMM(nobs))
 allocate (DaltGPS(nobs),DlatGPS(nobs),DlongGPS(nobs))
 allocate (QGPS(nobs),TedrGPS(nobs),ReynGPS(nobs))
 allocate (PStaMM(nobs),TstaMM(nobs),ThtaMM(nobs),UMM(nobs))
 allocate (VMM(nobs),WMM(nobs))

 j=1 ; do_rel=.false. ; stop_loop=.false.
 do while(.not. stop_loop)
   read(unitER2_o3,*,err=997,end=120) timeO3(j), O3(j)
   if(.not.do_rel) then
     if((timeO3(j)>(start_ER2time-0.9999)) &
     .and.(timeO3(j).le.start_ER2time)) then
        do_rel=.true. ; j=j+1
     else
        cycle
     endif
   else   
     if((timeO3(j)<(end_ER2time+0.9999)) &
     .and.(timeO3(j).ge.end_ER2time)) then
        stop_loop=.true.
     else
       j=j+1 ; if (j.gt.nobs) go to 995
     endif
   endif
 enddo
 nO3obs=j
 print *,'fixER2interp > O3 obs read ',nO3obs
 j=1 ; do_rel=.false. ; stop_loop=.false.
 do while(.not. stop_loop)
   read(unitER2_MG,*,err=997,end=120) UTGPS(j), &
     DaltGPS(j), DlatGPS(j), DLongGPS(j), QGPS(j), &
     TedrGPS(j), ReynGPS(j)
   if(.not.do_rel) then
     if((UTGPS(j)>(start_ER2time-0.9999)) &
     .and.(UTGPS(j).le.start_ER2time)) then
        do_rel=.true. ; j=j+1
     else
        cycle
     endif
   else   
     if((UTGPS(j)<(end_ER2time+0.9999)) &
     .and.(UTGPS(j).ge.end_ER2time)) then
        stop_loop=.true.
     else
       j=j+1 ; if (j.gt.nobs) go to 995
     endif
   endif
 enddo
 nGPSobs=j
 print *,'fixER2interp > GPS obs read ',nGPSobs
 j=1 ; do_rel=.false. ; stop_loop=.false.
 do while(.not. stop_loop)
   read(unitER2_MM,*,err=997,end=120) GMTMM(j), &
     PStaMM(j), TstaMM(j), ThtaMM(j), UMM(j), VMM(j), WMM(j)
   if(.not.do_rel) then
     if((GMTMM(j)>(start_ER2time-0.9999)) &
     .and.(GMTMM(j).le.start_ER2time)) then
        do_rel=.true. ; j=j+1
     else
        cycle
     endif
   else   
     if((GMTMM(j)<(end_ER2time+0.9999)) &
     .and.(GMTMM(j).ge.end_ER2time)) then
        stop_loop=.true.
     else
       j=j+1 ; if (j.gt.nobs) go to 995
     endif
   endif
 enddo
 nMMobs=j
 print *,'fixER2interp > Meteo obs read ',nMMobs

 allocate (F_z(nMMobs),F_lat(nGPSobs),F_long(nGPSobs))
 if(z_motion) & 
   F_z(1:nMMobs) = -log(10.*real(PStaMM(1:nMMobs))/p0) ! p0 in Pascal
 if(diabatic_w.or.isentropic_motion) &
   F_z(1:nMMobs) = ThtaMM(1:nMMobs) * 0.1
 F_lat(1:nGPSobs)  = real(DlatGPS(1:nGPSobs)) *1.e-5
 F_long(1:nGPSobs) = real(DlongGPS(1:nGPSobs)) *1.e-5
 
 if( activ_Lyapunov ) then
   numpart_np = n_loc
   numpart = numpart_np * 7
 else
   numpart = n_loc * n_sample
 endif
 error = (check_numpart() == 1).or.error
 
!  Interpolation
 print *,'fixER2interp > interpolating and filling positions'
 allocate(interp_time(n_loc))
 allocate(interp_xtra1(n_loc),interp_ytra1(n_loc),interp_ztra1(n_loc))
 do i=1,n_loc
   interp_time(i) = Start_ER2time+(i-1)*(End_ER2time-Start_ER2time)/(n_loc-1)
 enddo
! print *,julian_date(1),julian_date(2),julian_date(3)
! print *,off_jul(1),off_jul(nobs)
! write(*,'(a,2g12.5)') 'fixER2interp> interp_time ',interp_time(1),interp_time(n_loc)
 call uvip3p(3,nGPSobs,UTGPS,F_long,n_loc,interp_time,interp_xtra1)
 call uvip3p(3,nGPSobs,UTGPS,F_lat ,n_loc,interp_time,interp_ytra1)
 call uvip3p(3,nMMobs, GMTMM,F_z   ,n_loc,interp_time,interp_ztra1)
 j1=1   
 do i=1,n_loc
   itra1(j1) = int((juldate(FDate,000000)-bdate)*86400. + interp_time(i))
   error = (check_launch_time() == 1).or.error
   do j=j1,j1+n_sample-1
     itra1(j) = int((juldate(FDate,000000)-bdate)*86400. + interp_time(i))
     xtra1(j) = (interp_xtra1(i) - xlon0)/dx
     ytra1(j) = (interp_ytra1(i) - ylat0)/dy
     ztra1(j) = interp_ztra1(i)
     itramem(j) = itra1(j)
   enddo
   j1=j1+n_sample
 enddo
 
 print *,'fixER2interp> release end'
 print *,'fixER2interp> ',numpart
 write(*,'(a,2g12.5)') ' fixER2interp> itra1 ',itra1(1),itra1(numpart)
 write(*,'(a,2g12.5)') ' fixER2interp> xtra1 ',xtra1(1),xtra1(numpart)
 write(*,'(a,2g12.5)') ' fixER2interp> ytra1 ',ytra1(1),ytra1(numpart)  
 write(*,'(a,2g12.5)') ' fixER2interp> ztra1 ',ztra1(1),ztra1(numpart) 
 
 close(unitER2_O3)
 close(unitER2_MG)
 close(unitER2_MM)
 deallocate(timeO3, UTGPS, GMTMM,interp_time,F_z, F_lat, F_long, &
   O3, DaltGPS, DlatGPS, DLongGPS, QGPS, TedrGPS, ReynGPS, &
   PStaMM, TstaMM, ThtaMM, UMM, VMM, WMM, &
   interp_ztra1,interp_xtra1,interp_ytra1)
 return
                   

120   print *, 'fixER2interp> unexpected EOF'
      error=.true.
      print *, 'special diag ',j,GMTMM(j-1),GMTMM(j-2)
      return

995   error=.true.
      write(*,*) '#####################################################'
      write(*,*) '#### FLEXPART MODEL SUBROUTINE FIXER2SOLVEINTERP ####'
      write(*,*) '####                                             ####'
      write(*,*) '#### ERROR - NOBS TOO SMALL                      ####'
      write(*,*) '#####################################################'      
      return
996   error=.true.
      write(*,*) '#####################################################'
      write(*,*) '#### FLEXPART MODEL SUBROUTINE FIXER2SOLVEINTERP ####'
      write(*,*) '####                                             ####'
      write(*,*) '#### ERROR - FILE CANNOT BE OPENED               ####'
      write(*,*) '#####################################################'      
      return
997   error=.true.
      write(*,*) '#####################################################'
      write(*,*) '#### FLEXPART MODEL SUBROUTINE FIXER2SOLVEINTERP ####'
      write(*,*) '####                                             ####'
      write(*,*) '#### ERROR WHILE READING THE FILE                ####'
      write(*,*) '#####################################################'      
      return     

      return    
      
  contains  
    
     function check_launch_time()
     integer :: check_launch_time
     check_launch_time=0
     if(itra1(j1)>0) then
       write(*,*) ' #### FLEXPART MODEL ERROR! LAUNCH TIME >0    #### ' 
       write(*,*) i,itra1(j1)
       check_launch_time=1
     endif
     end function check_launch_time
     function check_numpart()
     integer :: check_numpart
     check_numpart=0
     if(numpart > maxpart) then
       write(*,*) '#####################################################'
       write(*,*) '#### FLEXPART MODEL SUBROUTINE FIXER2INTERP :    ####'
       write(*,*) '####                                             ####'
       write(*,*) '#### NB OF REQUIRED PARTICLES EXCEEDS MAXPART    ####'
       write(*,*) '#####################################################'             
       check_numpart=1
     endif
     end function check_numpart

  end subroutine fixER2SOLVEinterp
!
!=====|==1=========2=========3=========4=========5=========6=========7==
!
!$Log: 
