!$Id:
!
!=======================================================================
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ FLEXPART @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!=====|==1=========2=========3=========4=========5=========6=========7==
!
  subroutine fixmozaicinterp(error)
!*******************************************************************************
!                                                                              *
!     This routine fixes the release times and release locations of all        *
!     particles.
!     Interpolate selected part of the flight path to n_loc points regularly
!     distributed in time. Put n_sample particles at each point.                                                *
!
!     Changes: B. Legras, Jun. 2002
!              version for distribution along an aircraft transect 
!              with temporal interpolation
!                                                                              *
!*******************************************************************************
!                                                                              *
! Variables:                                                                   *
! ideltas [s]          simulation period                                       *
!                                                                              *
!*******************************************************************************

 use commons
 implicit none

 real, allocatable, dimension(:) :: F_lat,F_long,F_baroalt,F_radioalt, &
      F_press, A340_statictemp,A340_airspeed,F_groundspeed,F_zonalwind, &
      F_meridionalwind,F_ozone,F_statictemp,F_relhumid,F_RHvalid,F_RHaccur, &
      F_water,F_z,off_jul,interp_time, &
      interp_xtra1, interp_ytra1, interp_ztra1
 integer, allocatable, dimension (:) :: F_date, F_time 
 character(len=96):: line
 logical error,do_rel,stop_loop
 integer hh,mm,ss,hh_start,mm_start,ss_start,hh_end,mm_end,ss_end
 integer i,j,j1,rank_obs,nobs
 real diff_time
 double precision, allocatable :: julian_date(:)
 double precision juldate
 
 error=.false.
 open(unitmozaic,file=trim(MOZAIC_dir)//trim(MOZAIC_filename), &
      status='OLD',err=996,FORM='FORMATTED')
 
! read first 3 lines
 read(unitmozaic,*)
 read(unitmozaic,'(a)') line
 read(unitmozaic,*)
 print *,'fixmozaicinterp >',line
 
 select case (index_type)
 case ('rank')
   nobs = end_index - start_index +1
 case ('time')
   hh_start = start_index/10000
   mm_start = (start_index-10000*hh_start)/100
   ss_start = (start_index-10000*hh_start-100*mm_start)
   hh_end = end_index/10000
   mm_end = (end_index-10000*hh_end)/100
   ss_end = end_index-10000*hh_end-100*mm_end
   if(hh_end > hh_start) then 
      diff_time = (hh_end - hh_start)*3600
   else
      diff_time = (24+hh_end - hh_start)*3600
   endif
   diff_time=diff_time+(mm_end - mm_start)*60 + &
                        ss_end - ss_start
   nobs=int(diff_time/4 * 1.05)
   allocate (F_lat(nobs),F_long(nobs),F_baroalt(nobs),F_radioalt(nobs))
   allocate (F_press(nobs),A340_statictemp(nobs),A340_airspeed(nobs))
   allocate (F_groundspeed(nobs),F_zonalwind(nobs),F_meridionalwind(nobs))
   allocate (F_ozone(nobs),F_statictemp(nobs),F_relhumid(nobs),F_RHvalid(nobs))
   allocate (F_RHaccur(nobs),F_water(nobs),F_date(nobs),F_time(nobs))
   allocate (F_z(nobs),julian_date(nobs),off_jul(nobs))
 end select
 
 rank_obs=0
 j=1
 do_rel=.false. ; stop_loop=.false.
 do while(.not. stop_loop)
   read(unitmozaic,*,err=997,end=120) &
      F_date(j),F_time(j),F_lat(j),F_long(j),F_baroalt(j),F_radioalt(j),&
      F_press(j),A340_statictemp(j),A340_airspeed(j),F_groundspeed(j),  &
      F_zonalwind(j),F_meridionalwind(j),F_ozone(j),F_statictemp(j),    &
      F_relhumid(j),F_RHvalid(j),F_RHaccur(j),F_water(j)
   rank_obs=rank_obs+1
   select case (index_type)
   case ('rank')
      if(rank_obs < start_index)cycle
      if(rank_obs == end_index) then
        stop_loop=.true. ; cycle
      endif
      do_rel=.true.
      j=j+1
   case ('time')
      hh = F_time(j)/10000
      mm = (F_time(j)-10000*hh)/100
      ss = F_time(j)-10000*hh-100*mm
      if(.not.do_rel) then
        if((hh==hh_start).and.(mm==mm_start).and.(ss==ss_start)) then
          do_rel=.true.
          print *,'fixmozaicinterp> release start'
          j=j+1
        else
          cycle
        endif
      else   
        if((hh==hh_end).and.(mm==mm_end).and.(ss==ss_end)) then
          stop_loop=.true.
        else
          j=j+1
          if (j.gt.nobs) go to 995
      endif
      endif
   end select
 enddo
 
 nobs=j
 print *,'fixmozaicinterp > nobs ',nobs
 do j=1,nobs
   julian_date(j)=juldate(F_date(j),F_time(j))
   F_z(j) = -log(F_press(j)/p0)
 enddo
 do j=1,nobs
   off_jul(j)=julian_date(j)-julian_date(1)
 enddo
 
 numpart = n_loc * n_sample
 error = (check_numpart() == 1).or.error
 
!  Interpolation
 print *,'fixmozaicinterp > interpolating and filling positions' 
 allocate(interp_time(n_loc))
 allocate(interp_xtra1(n_loc),interp_ytra1(n_loc),interp_ztra1(n_loc))
 do i=1,n_loc
   interp_time(i) = (i-1)*(julian_date(nobs)-julian_date(1))/(n_loc-1)
 enddo
 print *,julian_date(1),julian_date(2),julian_date(3)
 print *,off_jul(1),off_jul(nobs)
 print *,interp_time(1),interp_time(n_loc)
 call uvip3p(3,nobs,off_jul,F_long,n_loc,interp_time,interp_xtra1)
 call uvip3p(3,nobs,off_jul,F_lat ,n_loc,interp_time,interp_ytra1)
 call uvip3p(3,nobs,off_jul,F_z   ,n_loc,interp_time,interp_ztra1)
 j1=1
 do i=1,n_loc
   itra1(j1) = int((julian_date(1)-bdate & 
            +(i-1)*(julian_date(nobs)-julian_date(1))/(n_loc-1))*86400)
   error = (check_launch_time() == 1).or.error
   do j=j1,j1+n_sample-1
     itra1(j) = int((julian_date(1)-bdate & 
            +(i-1)*(julian_date(nobs)-julian_date(1))/(n_loc-1))*86400)
     xtra1(j) = (interp_xtra1(i) - xlon0)/dx
     ytra1(j) = (interp_ytra1(i) - ylat0)/dy
     ztra1(j) = interp_ztra1(i)
     itramem(j) = itra1(j)
   enddo
   j1=j1+n_sample
 enddo
 
 print *,'fixmozaicinterp> release end'
 print *,'fixmozaicinterp> ',numpart
 print *,'fixmozaicinterp> F_long(obs) ',F_long(1),F_long(nobs)
 print *,'fixmozaicinterp>             ',xtra1(1)*dx + xlon0,xtra1(numpart)*dx + xlon0
 print *,'fixmozaicinterp> F_lat(obs)  ',F_lat(1),F_lat(nobs)
 print *,'fixmozaicinterp>             ',ytra1(1)*dy + ylat0,ytra1(numpart)*dy + ylat0
 print *,'fixmozaicinterp> F_z  (obs)  ',F_z(1),F_z(nobs)
 print *,'fixmozaicinterp>             ',ztra1(1),ztra1(numpart)
 
 close(unitmozaic)
 deallocate(F_lat,F_long,F_baroalt,F_radioalt, &
      F_press, A340_statictemp,A340_airspeed,F_groundspeed,F_zonalwind, &
      F_meridionalwind,F_ozone,F_statictemp,F_relhumid,F_RHvalid,F_RHaccur, &
      F_water,F_time,F_date,F_z,interp_time,julian_date,off_jul, &
      interp_ztra1,interp_xtra1,interp_ytra1)
 return                  

120   print *, 'fixmozaicpart> unexpected EOF'
      error=.true.
      return

995   error=.true.
      write(*,*) '#####################################################'
      write(*,*) '#### FLEXPART MODEL SUBROUTINE FIXMOZAICPART:    ####'
      write(*,*) '####                                             ####'
      write(*,*) '#### ERROR - NOBS TOO SMALL                      ####'
      write(*,*) '#####################################################'      
      return
996   error=.true.
      write(*,*) '#####################################################'
      write(*,*) '#### FLEXPART MODEL SUBROUTINE FIXMOZAICPART:    ####'
      write(*,*) '####                                             ####'
      write(*,*) '#### ERROR - FILE CANNOT BE OPENED               ####'
      write(*,*) '#####################################################'      
      return
997   error=.true.
      write(*,*) '#####################################################'
      write(*,*) '#### FLEXPART MODEL SUBROUTINE FIXMOZAICPART:    ####'
      write(*,*) '####                                             ####'
      write(*,*) '#### ERROR WHILE READING THE FILE                ####'
      write(*,*) '#####################################################'      
      return     
      
 contains
 
     function check_launch_time()
     integer :: check_launch_time
     check_launch_time=0
     if(itra1(j1)>0) then
       write(*,*) ' #### FLEXPART MODEL ERROR! LAUNCH TIME >0    #### ' 
       write(*,*) i,itra1(j1)
       check_launch_time=1
     endif
     end function check_launch_time
     function check_numpart()
     integer :: check_numpart
     check_numpart=0
     if(numpart > maxpart) then
       write(*,*) '#####################################################'
       write(*,*) '#### FLEXPART MODEL SUBROUTINE FIXMOZAICINTERP : ####'
       write(*,*) '####                                             ####'
       write(*,*) '#### NB OF REQUIRED PARTICLES EXCEEDS MAXPART    ####'
       write(*,*) '#####################################################'             
       check_numpart=1
     endif
     end function check_numpart

  end subroutine fixmozaicinterp  
!
!=====|==1=========2=========3=========4=========5=========6=========7==
!
!$Log: 
