!$Id:
!
!=======================================================================
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ FLEXPART @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!=====|==1=========2=========3=========4=========5=========6=========7==
!
  subroutine fixmozaicpart(error)
!*******************************************************************************
!                                                                              *
!     This routine fixes the release times and release locations of all        *
!     particles.           						       *
!     Put n_sample particles at each measurement point of the selected     
!     part of the flight
!
!     Changes: B. Legras, Apr. 2002
!              version for distribution along a profile
!                                                                              *
!*******************************************************************************
!                                                                              *
! Variables:                                                                   *
! ideltas [s]          simulation period                                       *
!                                                                              *
!*******************************************************************************

 use commons
 implicit none

 real F_lat,F_long,F_baroalt,F_radioalt,F_press, &
      A340_statictemp,A340_airspeed,F_groundspeed,F_zonalwind, &
      F_meridionalwind,F_ozone,F_statictemp,F_relhumid,F_RHvalid,F_RHaccur, &
      F_water
 integer F_date,F_time 
 character(len=96):: line
 logical error,do_rel,stop_loop
 integer hh,mm,ss,hh_start,mm_start,ss_start,hh_end,mm_end,ss_end
 integer i,rank_obs,itra_part
 real x_part,y_part
 double precision julian_date
 double precision juldate
 
 error=.false.
 open(unitmozaic,file=trim(MOZAIC_dir)//trim(MOZAIC_filename), &
      status='OLD',err=996,FORM='FORMATTED')
 
! read first 3 lines
 read(unitmozaic,*)
 read(unitmozaic,'(a)') line
 read(unitmozaic,*)
 print *,'fixmozaicpart >',line
 
 if(index_type=='time') then
   hh_start = start_index/10000
   mm_start = (start_index-10000*hh_start)/100
   ss_start = (start_index-10000*hh_start-100*mm_start)
   hh_end = end_index/10000
   mm_end = (end_index-10000*hh_end)/100
   ss_end = end_index-10000*hh_end-100*mm_end
 endif  
 
 rank_obs=0
 do_rel=.false. ; stop_loop=.false.
 numpart=0
 do while(.not. stop_loop)
   read(unitmozaic,*,err=997,end=120) &
      F_date,F_time,F_lat,F_long,F_baroalt,F_radioalt,F_press,&
      A340_statictemp,A340_airspeed,F_groundspeed,F_zonalwind,F_meridionalwind,&
      F_ozone,F_statictemp,F_relhumid,F_RHvalid,F_RHaccur,F_water
   rank_obs=rank_obs+1
   select case (index_type)
   case ('rank')
      if(rank_obs < start_index)cycle
      if(rank_obs > end_index) then
        stop_loop=.true. ; cycle
      endif
      do_rel=.true.
   case ('time')
      hh = F_time/10000
      mm = (F_time-10000*hh)/100
      ss = F_time-10000*hh-100*mm
      if(.not.do_rel) then
        if((hh==hh_start).and.(mm==mm_start).and.(ss==ss_start)) then
          do_rel=.true.
          print *,'fixmozaicpart> release start'
        else
          cycle
        endif
      else if((hh==hh_end).and.(mm==mm_end).and.(ss==ss_end)) then
        stop_loop=.true.
      endif
   end select
   
   if(.not.do_rel) cycle
   julian_date=juldate(F_date,F_time)
   if(instant_release) then
      itra_part=int((julian_date-bdate)*86400)
   else
      itra_part=sign(int(abs(julian_date-bdate)*86400/abs(lsynctime)+0.5) &
         *lsynctime,int((julian_date-bdate)*86400))
   endif
   x_part=(F_long-xlon0)/dx
   y_part=(F_lat-ylat0)/dy
!  print *,'fixmozaic> ',F_date,F_time,itra_part
   if(numpart+n_sample > maxpart) go to 998
   do i=numpart+1,numpart+n_sample
      itra1(i) = itra_part
      xtra1(i) = x_part
      ytra1(i) = y_part
      ztra1(i) = -log(F_press/p0)
      itramem(i) = itra_part                       
   enddo
   numpart=numpart+n_sample
 enddo
 print *,'fixmozaicpart> release end'
 print *,'fixmozaicpart> ',numpart
 close(unitmozaic)
 return                  

120   print *, 'fixmozaicpart> unexpected EOF'
      error=.true.
      return

996   error=.true.
      write(*,*) '#####################################################'
      write(*,*) '#### FLEXPART MODEL SUBROUTINE FIXMOZAICPART:    ####'
      write(*,*) '####                                             ####'
      write(*,*) '#### ERROR - FILE CANNOT BE OPENED               ####'
      write(*,*) '#####################################################'      
      return
997   error=.true.
      write(*,*) '#####################################################'
      write(*,*) '#### FLEXPART MODEL SUBROUTINE FIXMOZAICPART:    ####'
      write(*,*) '####                                             ####'
      write(*,*) '#### ERROR WHILE READING THE FILE                ####'
      write(*,*) '#####################################################'      
      return     
998   error=.true.
      write(*,*) '#####################################################'
      write(*,*) '#### FLEXPART MODEL SUBROUTINE FIXMOZAICPART:    ####'
      write(*,*) '####                                             ####'
      write(*,*) '#### NUMBER OF REQUIRED POINTS EXCEEDS MAXPART   ####'
      write(*,*) '#####################################################'      
      return          

      end
!
!=====|==1=========2=========3=========4=========5=========6=========7==
!
!$Log: 
