!$Id:
!
!=======================================================================
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ FLEXPART @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!=====|==1=========2=========3=========4=========5=========6=========7==
!
      subroutine fixmultilayer(error)
!*******************************************************************************
!                                                                              *
!     This routine fixes the release times and release locations of all        *
!     particles a fixed number of potential temperature layers.                *
!                                                                              *
!     Author: B. Legras
!                                                                              *
!    
!                                                                              *
!*******************************************************************************
!                                                                              *
! Variables:                                                                   *
!                                                                              *
!*******************************************************************************

      use commons
      implicit none

      real xc,yc,zc, epsil,epsil2, ylat
      real corrected_mesh_size
      integer i,ix,jy,it,it_start,it_end,count_lon,count_lat,count_layer
      integer nb_points_latcircle
      logical error

      error = .false.
      epsil =  1.e-4 ; epsil2=1.e-7

      allocate (itra0(maxpart))
      
      numpart=0 
      print *,'numpoint ',numpoint
      print *,'path',path(2)
      mesh_size_lat  = mesh_size_lat/dy
      mesh_size_long = mesh_size_long/dx
      print *,'fixmultilayer > meshs ',mesh_size_long,mesh_size_lat
      print *,'fixmultilayer > xlon0,ylat0,dx,dy ',xlon0,ylat0,dx,dy
!     loop on the number of layers
      dolayer: do i=1,numpoint
        print *,'fixmultilayer  layer ',i
        zc=zpoint1(i)
        xpoint1(i) = (xpoint1(i)-xlon0)/dx
        xpoint2(i) = (xpoint2(i)-xlon0)/dx
        ypoint1(i) = (ypoint1(i)-ylat0)/dy
        ypoint2(i) = (ypoint2(i)-ylat0)/dy
!       define time to be on a time-step boundary       
        it = ireleasestart(i) - mod(ireleasestart(i),abs(lsynctime))
        print *,'fixmultilayer > zc     ',zc
        print *,'fixmultilayer > xpoint ',xpoint1(i),xpoint2(i)
        print *,'fixmultilayer > ypoint ',ypoint1(i),ypoint2(i)
        if ((xpoint1(i) < 0.).or.(xpoint1(i) > nx+epsil2)) go to 999
        if ((xpoint2(i) < 0.).or.(xpoint2(i) > nx+epsil2)) go to 999
        if ((ypoint1(i) < 0.).or.(ypoint1(i) > ny+epsil2)) go to 999
        if ((ypoint2(i) < 0.).or.(ypoint2(i) > ny+epsil2)) go to 999
!       define start and end times to be on a time-step boundary       
        it_start = ireleasestart(i) - mod(ireleasestart(i),abs(lsynctime))
        it_end   = ireleaseend(i)   - mod(ireleaseend(i),  abs(lsynctime))
!       check that the release interval is a divider of end - start
        if (mod(it_start-it_end,ireleaseinterval(i)).ne.0) go to 995
!       loop on time
        dotime: do it=it_start,it_end,ireleaseinterval(i)
!         loop on latitude
          yc = ypoint1(i)
          dolat: do while (yc <= ypoint2(i)+epsil)
            jy=floor(yc)
            ylat = (ylat0 + yc*dy)*pi/180.
!           defined corrected mesh if needed           
            if (uniform_mesh.and.uniform_spread) then
              nb_points_latcircle = int(nx*abs(cos(ylat))/mesh_size_long)
              if(nb_points_latcircle>0) then
                corrected_mesh_size = float(nx)/nb_points_latcircle
              else 
                corrected_mesh_size = 2*float(nx) ! set a large value to get only 1 pt on circle
              endif
            else if (uniform_spread) then
              corrected_mesh_size = mesh_size_long/(abs(cos(ylat))+epsil2)
            else
              corrected_mesh_size = mesh_size_long
            endif
!           loop in longitude
            xc = xpoint1(i)
            dolong: do while (xc <= xpoint2(i)+epsil)
              ix=floor(xc)
              numpart=numpart+1
              if(numpart > maxpart) go to 996
              xtra1(numpart) = xc
              ytra1(numpart) = yc
              ztra1(numpart) = zc
              itra1(numpart) = it
              itra0(numpart) = it
              xc = xc + corrected_mesh_size
            enddo dolong
            yc = yc + mesh_size_lat
          enddo dolat
        enddo dotime
      enddo dolayer
      open(unitpartout4,file=trim(path(2))//'release_table_5')
!     loop to make a release table (the same for each time)
      write(unitpartout4,*)'plan: AGEF'
      write(unitpartout4,'(A,I3)')'nb_layer: ',numpoint
      dolayer2: do i=1,numpoint
        count_layer=0
        count_lat=0
        yc = ypoint1(i)
        do while (yc <= ypoint2(i)+epsil)
          count_lat=count_lat+1
          yc=yc+mesh_size_lat
        enddo
        write(unitpartout4,'(A,I3,I4)')'layer,nb_lat: ',i,count_lat
        yc = ypoint1(i)
        dolat2: do while (yc <= ypoint2(i)+epsil)
          jy=floor(yc)
          ylat = (ylat0 + yc*dy)*pi/180.
          if (uniform_mesh.and.uniform_spread) then
             nb_points_latcircle = int(nx*abs(cos(ylat))/mesh_size_long)
             if(nb_points_latcircle>0) then
               corrected_mesh_size = float(nx)/nb_points_latcircle
             else 
               corrected_mesh_size = 2*float(nx) ! set a large value to get only 1 pt on circle
             endif
          else if (uniform_spread) then
              corrected_mesh_size = mesh_size_long/(abs(cos(ylat))+epsil2)
          else
              corrected_mesh_size = mesh_size_long
          endif
          xc = xpoint1(i)
          count_lon=0
          dolong2: do while (xc <= xpoint2(i)+epsil)
            ix=floor(xc)
            count_lon=count_lon+1           
            xc = xc + corrected_mesh_size
            count_layer=count_layer+1
          enddo dolong2
          write(unitpartout4,'(3G15.6,I6)') ylat0+yc*dx,xlon0,corrected_mesh_size*dx,count_lon
          yc = yc + mesh_size_lat
        enddo dolat2
        write(unitpartout4,'(A,I3,I6)')'layer,count_layer: ',i,count_layer
      enddo dolayer2
      close(unitpartout4)
!     Save the initial positions
      open(unitpartout4,file=trim(path(2))//'sav_init',form='unformatted')
      write(unitpartout4) xtra1(1:numpart)
      write(unitpartout4) ytra1(1:numpart)
      write(unitpartout4) ztra1(1:numpart)
      write(unitpartout4) itra0(1:numpart)
      close(unitpartout4)
      print *,'fixmultilayer > completed, count_layer, numpart ',count_layer, numpart
      return

996   error=.true.
      print *,'numpart ',numpart
      print *,'xc yc ',xc,yc
      write(*,*) '#####################################################'
      write(*,*) '#### FLEXPART MODEL SUBROUTINE FIXMULTILAYER:    ####'
      write(*,*) '####                                             ####'
      write(*,*) '#### ERROR - TOTAL NUMBER OF PARTICLES SPECIFIED ####'
      write(*,*) '#### IN FILE "RELEASES" OR CARRIED FORWARD FROM  ####'
      write(*,*) '#### PREVIOUS RUN EXCEEDS THE MAXIMUM ALLOWED    ####'
      write(*,*) '#### NUMBER. REDUCE EITHER NUMBER OF PARTICLES   ####'
      write(*,*) '#### PER RELEASE POINT OR REDUCE NUMBE OF        ####'
      write(*,*) '#### RELEASE POINTS.                             ####'
      write(*,*) '#####################################################'
      return

999   error=.true.
      write(*,*) '#####################################################'
      write(*,*) '###  ERROR IN THE XPOINT YPOINT INITIALIZATION    ###'
      write(*,*) '#####################################################'
      return

995   error=.true.
      write(*,*) '#####################################################'
      write(*,*) '###  ERROR THE LAUNCH INTERVAL IS NOT A DIVIDER   ###'
      write(*,*) '###  OF THE START - END PERIOD                    ###'
      write(*,*) '#####################################################'
      return

      end subroutine fixmultilayer

 
!=====|==1=========2=========3=========4=========5=========6=========7==
!
!$Log: 
