!$Id: 
!
!=======================================================================
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ FLEXPART @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!=====|==1=========2=========3=========4=========5=========6=========7==
!
      subroutine getfields(itime,nstop)
!                            i     o
!******************************************************************************
!                                                                              *
!  This subroutine manages the 3 data fields to be kept in memory.             *
!  During the first time step of petterssen it has to be fulfilled that the    *
!  first data field must have |wftime|<itime, i.e. the absolute value of wftime*
!  must be smaller than the absolute value of the current time in [s].         *
!  The other 2 fields are the next in time after the first one.                *
!  Pointers (memind) are used, because otherwise one would have to resort the  *
!  wind fields, which costs a lot of computing time. Here only the pointers are*
!  resorted.                                                                   *
!                                                                              *
!     Author: A. Stohl                                                         *
!                                                                              *
!     29 April 1994                                                            *
!                                                                              *
!*******************************************************************************
!  Changes, Bernd C. Krueger, Feb. 2001:
!        Variables tth,qvh,tthn,qvhn (on eta coordinates) in common block.
!        Function of nstop extended.
!           B. Legras, April 2002:
!        Nested operations cancelled.
!        uuh, vvh, wwh on two times and in common
!        Message printed when reading a field
!
!*******************************************************************************
!                                                                              *
! Variables:                                                                   *
! lwindinterval [s]    time difference between the two wind fields read in     *
! indj                 indicates the number of the wind field to be read in    *
! indmin               remembers the number of wind fields already treated     *
! memind(2)            pointer, on which place the wind fields are stored      *
! memtime(2) [s]       times of the wind fields, which are kept in memory      *
! itime [s]            current time since start date of trajectory calculation *
! nstop                > 0, if trajectory has to be terminated                 *
! nx,ny,nuvz,nwz       field dimensions in x,y and z direction                 *
! uuh(0:nxmax,0:nymax,nuvzmax,2)   wind components in x-direction [m/s]         *
! vvh(0:nxmax,0:nymax,nuvzmax,2)   wind components in y-direction [m/s]         *
! wwh(0:nxmax,0:nymax,nwzmax,2)    wind components in z-direction [deltaeta/s]  *
!                                                                              *
! Constants:                                                                   *
! idiffmax             maximum allowable time difference between 2 wind fields *
!                                                                            *
!*******************************************************************************

      use commons
      implicit none

      integer indj,indmin,itime,nstop,memaux
      save indmin

      data indmin/1/

 
! Check, if wind fields are available for the current time step
!**************************************************************

      nstop=0

      if ((ldirect*wftime(1).gt.ldirect*itime).or.    &
        (ldirect*wftime(numbwf).lt.ldirect*itime)) then
        write(*,*) 'FLEXPART WARNING: NO WIND FIELDS ARE AVAILABLE.'
        write(*,*) 'A TRAJECTORY HAS TO BE TERMINATED.'
        nstop=4
        return
      endif


      if ((ldirect*memtime(1).le.ldirect*itime).and.   &
        (ldirect*memtime(2).gt.ldirect*itime)) then

! The right wind fields are already in memory -> don't do anything
!*****************************************************************

        continue

      else if ((ldirect*memtime(2).le.ldirect*itime).and. &
        (memtime(2).ne.999999999)) then
 

! Current time is after 2nd wind field
! -> Resort wind field pointers, so that current time is between 1st and 2nd
!***************************************************************************

        memaux=memind(1)
        memind(1)=memind(2)
        memind(2)=memaux
        memtime(1)=memtime(2)

! Read a new wind field and store it on place memind(2)
!******************************************************
! to do: rewrite this with a WHILE
       
        do indj=indmin,numbwf-1
           if (ldirect*wftime(indj+1).gt.ldirect*itime) then
              call readwind(indj+1,memind(2))
!             call calcpar(memind(2))
              memtime(2)=wftime(indj+1)
              call verttransformB(memind(2))
              write(*,'(a,a,a,i11,a,i11)') &
                      ' getfields> file ',trim(wfname(indj+1)),&
                      '  memtime ',memtime(2),'  time ',itime
              nstop = 1
              goto 40
           endif
       enddo
       go to 90
 40    indmin=indj ! memorize last value read

      else

! No wind fields, which can be used, are currently in memory 
! -> read both wind fields
!***********************************************************

         do indj=indmin,numbwf-1
            if ((ldirect*wftime(indj).le.ldirect*itime).and.   &
                  (ldirect*wftime(indj+1).gt.ldirect*itime)) then
               memind(1)=1
               call readwind(indj,memind(1))
!              call calcpar(memind(1))
               memtime(1)=wftime(indj)
               write(*,'(a,a,a,i11,a,i11)') &
                      ' getfields> 1st  ',trim(wfname(indj)),&
                      '  memtime ',memtime(1),'  time ',itime
!               write(*,'(a,a,a,i11,i10,i8)') &
!                      ' getfields> 1st  ',trim(wfname(indj)),&
!                      '  time ',memtime(1),wfldat(indj),wfltim(indj)
               memind(2)=2
               call readwind(indj+1,memind(2))
!              call calcpar(memind(2))
               memtime(2)=wftime(indj+1)
               call verttransformB(memind(1)) ! needs Ps next time
               call verttransformB(memind(2))
               write(*,'(a,a,a,i11,a,i11)') &
                      ' getfields> 2nd  ',trim(wfname(indj+1)),&
                      '  memtime ',memtime(2),'  time ',itime
!               write(*,'(a,a,a,i11,i10,i8)') &
!                      ' getfields> 2nd  ',trim(wfname(indj+1)),&
!                      '  time ',memtime(2),wfldat(indj+1),wfltim(indj+1)
               nstop = 1
               goto 60
            endif
         enddo
         go to 90
 60      indmin=indj

      endif

      lwindinterv=abs(memtime(2)-memtime(1))
 
      if (lwindinterv.gt.idiffmax) nstop=3

      return

 90   print *,'getfields > ERROR '
      print *,'TRY TO GET FIELD BEYOND AVAILABLE LIST'
      print *,'last ',wfname(numbwf),wfldat(numbwf), &
                      wfltim(numbwf),wftime(numbwf)
      print *,'called ',itime
      stop
      
      end
!
!=====|==1=========2=========3=========4=========5=========6=========7==
!
!$Log: 
