!$Id:
!
!=======================================================================
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ FLEXPART @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!=====|==1=========2=========3=========4=========5=========6=========7==
!
      subroutine interpol_windB   &
         (itime,xt,yt,zt,dxdt,dydt,dzdt,ngrid,psaver,z_factor,tint)

!*******************************************************************************
!                                                                              *
!  This subroutine interpolates the wind data to current trajectory position.  *
!                                                                              *
!    Author: A. Stohl                                                          *
!                                                                              *
!    16 December 1997  
! 
!    Changes: B. Legras, April 2002
!             interpolation from eta winds
!             variance calculation cancelled
!             B. Legras, June 2002
!             optimisation of log calculations
!             new calculation of z_factor for z and theta diffusion
!             B. Legras, December 2007
!             patch to avoid extrapolations at the top and bottom         
!                                                                              *
!*******************************************************************************
!                                                                              *
! Variables:                                                                   *
! dxdt,dydt,dzdt     wind components in grid units per second                  *
! itime [s]          current temporal position                                 *
! memtime(3) [s]     times of the wind fields in memory                        *
! xt,yt,zt           coordinates position for which wind data shall be calculat*
!                                                                              *
! Constants:                                                                   *
!                                                                              *
!*******************************************************************************

      use commons
      implicit none

      integer, intent(in) :: itime,ngrid
      real, intent(in) :: xt,yt,zt
      real, intent(out) :: dxdt,dydt,dzdt,psaver,z_factor,tint

! Auxiliary variables needed for interpolation
      real u1(2),v1(2),w1(2),dt1,dt2,dtt,pint,dTdLogp,tp1(2)
      real psl(4,2),pr(4,2),prp(4,2),u(4,2),v(4,2),w(4,2),tp(4,2)
      real pint1, pint2, pint3, pint4
      integer i,m,indexh,indz(4,2)
      integer ix,jy,ixp,jyp
      real ddx,ddy,rddx,rddy,p1,p2,p3,p4

!********************************************
! Multilinear interpolation in time and space
!********************************************

! Determine the lower left corner and its distance to the current position
!*************************************************************************

      ix=min(floor(xt),nx-2)       ;  jy=min(floor(yt),ny-2)
      ixp=ix+1         ;  jyp=jy+1
      ddx=xt-float(ix) ;  ddy=yt-float(jy)
      rddx=1.-ddx      ;  rddy=1.-ddy
      p1=rddx*rddy     ;  p2=ddx*rddy
      p3=rddx*ddy      ;  p4=ddx*ddy
      psl(1,1)=ps(ix ,jy ,1,memind(1))
      psl(1,2)=ps(ix ,jy ,1,memind(2))
      psl(2,1)=ps(ixp,jy ,1,memind(1))
      psl(2,2)=ps(ixp,jy ,1,memind(2))
      psl(3,1)=ps(ix ,jyp,1,memind(1))
      psl(3,2)=ps(ix ,jyp,1,memind(2))
      psl(4,1)=ps(ixp,jyp,1,memind(1))
      psl(4,2)=ps(ixp,jyp,1,memind(2))

! Calculate variables for time interpolation
!*******************************************

      dt1=float(itime-memtime(1))
      dt2=float(memtime(2)-itime)
      dtt=1./(dt1+dt2)

! Ground pressure at the location of the particle
!************************************************

      psaver = (dt2*(p1*psl(1,1)+p2*psl(2,1)+p3*psl(3,1)+p4*psl(4,1))   &
             +  dt1*(p1*psl(1,2)+p2*psl(2,2)+p3*psl(3,2)+p4*psl(4,2)))  &
             *  dtt

! Determine the level below the current position for u,v
!*******************************************************

!  Locates lower left corner

      pint = p0*exp(-zt)
      indz(1,1) = locuv(1,nuvz,psl(1,1))

!  Locates other points by assuming there are close to the first

      do i=2,4      
        indz(i,1)=locuv2(indz(1,1),psl(i,1))
      enddo
      do i=1,4
        indz(i,2)=locuv2(indz(1,1),psl(i,2))
      enddo

      do i=1,4
        pr (i,1)=akz(indz(i,1)  )+bkz(indz(i,1)  )*psl(i,1)
        pr (i,2)=akz(indz(i,2)  )+bkz(indz(i,2)  )*psl(i,2)
        prp(i,1)=akz(indz(i,1)+1)+bkz(indz(i,1)+1)*psl(i,1)
        prp(i,2)=akz(indz(i,2)+1)+bkz(indz(i,2)+1)*psl(i,2)
      enddo
!      if(jp==1) then 
!         print *,'interpol'
!         print *,pint
!         print *,(indz(i,1),i=1,4)
!         print *,(indz(i,2),i=1,4)
!         print *,(psl(i,1),i=1,4)
!         print *,(psl(i,2),i=1,4)
!      endif
!     Patch to avoid extrapolation in the boundary layer near orography
!     and at the top of the atmosphere      
      pint1=min(pint,min(pr(1,1),pr(1,2)))
      pint2=min(pint,min(pr(2,1),pr(2,2)))
      pint3=min(pint,min(pr(3,1),pr(3,2)))
      pint4=min(pint,min(pr(4,1),pr(4,2)))
      if(maxval(indz)==nuvz-1) pint=max(pint,maxval(prp)) ! partial patch
     

!**********************************************************************
! 1.) Bilinear horizontal interpolation
! This has to be done separately for 6 fields (Temporal(2)*Vertical(3))
!**********************************************************************

! Loop over 2 time steps and 2 levels
!************************************

      if (ngrid < 0) then
      
      select case(vert_interpol)
      
      case('log')
            
        do m=1,2
          indexh=memind(m)
            
            u(1,m)=(uupol(ix ,jy ,indz(1,m)  ,indexh)*(log(pint1)-log(prp(1,m))) &
                  + uupol(ix ,jy ,indz(1,m)+1,indexh)*(log(pr(1,m))-log(pint1))) &
                  / (log(pr(1,m))-log(prp(1,m))) 
            v(1,m)=(vvpol(ix ,jy ,indz(1,m)  ,indexh)*(log(pint1)-log(prp(1,m))) &
                  + vvpol(ix ,jy ,indz(1,m)+1,indexh)*(log(pr(1,m))-log(pint1))) &
                  / (log(pr(1,m))-log(prp(1,m)))
            u(2,m)=(uupol(ixp,jy ,indz(2,m)  ,indexh)*(log(pint2)-log(prp(2,m))) &
                  + uupol(ixp,jy ,indz(2,m)+1,indexh)*(log(pr(2,m))-log(pint2))) &
                  / (log(pr(2,m))-log(prp(2,m)))
            v(2,m)=(vvpol(ixp,jy ,indz(2,m)  ,indexh)*(log(pint2)-log(prp(2,m))) &
                  + vvpol(ixp,jy ,indz(2,m)+1,indexh)*(log(pr(2,m))-log(pint2))) &
                  / (log(pr(2,m))-log(prp(2,m)))
            u(3,m)=(uupol(ix ,jyp,indz(3,m)  ,indexh)*(log(pint3)-log(prp(3,m))) &
                  + uupol(ix ,jyp,indz(3,m)+1,indexh)*(log(pr(3,m))-log(pint3))) &
                  / (log(pr(3,m))-log(prp(3,m)))
            v(3,m)=(vvpol(ix ,jyp,indz(3,m)  ,indexh)*(log(pint3)-log(prp(3,m))) &
                  + vvpol(ix ,jyp,indz(3,m)+1,indexh)*(log(pr(3,m))-log(pint3))) &
                  / (log(pr(3,m))-log(prp(3,m)))
            u(4,m)=(uupol(ixp,jyp,indz(4,m)  ,indexh)*(log(pint4)-log(prp(4,m))) &
                  + uupol(ixp,jyp,indz(4,m)+1,indexh)*(log(pr(4,m))-log(pint4))) &
                  / (log(pr(4,m))-log(prp(4,m)))
            v(4,m)=(vvpol(ixp,jyp,indz(4,m)  ,indexh)*(log(pint4)-log(prp(4,m))) &
                  + vvpol(ixp,jyp,indz(4,m)+1,indexh)*(log(pr(4,m))-log(pint4))) &
                  / (log(pr(4,m))-log(prp(4,m)))
          
          u1(m)=p1*u(1,m)+p2*u(2,m)+p3*u(3,m)+p4*u(4,m)
          v1(m)=p1*v(1,m)+p2*v(2,m)+p3*v(3,m)+p4*v(4,m)

        enddo
        
      case('lin')
      
        do m=1,2
          indexh=memind(m)
            
            u(1,m)=(uupol(ix ,jy ,indz(1,m)  ,indexh)*(pint1-prp(1,m))  &
                  + uupol(ix ,jy ,indz(1,m)+1,indexh)*(pr(1,m)-pint1))  &
                  / (pr(1,m)-prp(1,m))
            v(1,m)=(vvpol(ix ,jy ,indz(1,m)  ,indexh)*(pint1-prp(1,m))  &
                  + vvpol(ix ,jy ,indz(1,m)+1,indexh)*(pr(1,m)-pint1))  &
                  / (pr(1,m)-prp(1,m))
            u(2,m)=(uupol(ixp,jy ,indz(2,m)  ,indexh)*(pint2-prp(2,m))  &
                  + uupol(ixp,jy ,indz(2,m)+1,indexh)*(pr(2,m)-pint2))  &
                  / (pr(2,m)-prp(2,m))
            v(2,m)=(vvpol(ixp,jy ,indz(2,m)  ,indexh)*(pint2-prp(2,m))  &
                  + vvpol(ixp,jy ,indz(2,m)+1,indexh)*(pr(2,m)-pint2))  &
                  / (pr(2,m)-prp(2,m))
            u(3,m)=(uupol(ix ,jyp,indz(3,m)  ,indexh)*(pint3-prp(3,m))  &
                  + uupol(ix ,jyp,indz(3,m)+1,indexh)*(pr(3,m)-pint3))  &
                  / (pr(3,m)-prp(3,m))
            v(3,m)=(vvpol(ix ,jyp,indz(3,m)  ,indexh)*(pint3-prp(3,m))  &
                  + vvpol(ix ,jyp,indz(3,m)+1,indexh)*(pr(3,m)-pint3))  &
                  / (pr(3,m)-prp(3,m))
            u(4,m)=(uupol(ixp,jyp,indz(4,m)  ,indexh)*(pint4-prp(4,m))  &
                  + uupol(ixp,jyp,indz(4,m)+1,indexh)*(pr(4,m)-pint4))  &
                  / (pr(4,m)-prp(4,m))
            v(4,m)=(vvpol(ixp,jyp,indz(4,m)  ,indexh)*(pint4-prp(4,m))  &
                  + vvpol(ixp,jyp,indz(4,m)+1,indexh)*(pr(4,m)-pint4))  &
                  / (pr(4,m)-prp(4,m))
          
          u1(m)=p1*u(1,m)+p2*u(2,m)+p3*u(3,m)+p4*u(4,m)
          v1(m)=p1*v(1,m)+p2*v(2,m)+p3*v(3,m)+p4*v(4,m)
          
        enddo
      
      end select 
                
      else
      
      select case(vert_interpol)
      
      case('log')      
      
        do m=1,2
          indexh=memind(m)

            u(1,m)=(uuh(ix ,jy ,indz(1,m)  ,indexh)*(log(pint1)-log(prp(1,m))) & 
                  + uuh(ix ,jy ,indz(1,m)+1,indexh)*(log(pr(1,m))-log(pint1))) &
                  / (log(pr(1,m))-log(prp(1,m)))
            v(1,m)=(vvh(ix ,jy ,indz(1,m)  ,indexh)*(log(pint1)-log(prp(1,m))) &
                  + vvh(ix ,jy ,indz(1,m)+1,indexh)*(log(pr(1,m))-log(pint1))) &
                  / (log(pr(1,m))-log(prp(1,m)))
            u(2,m)=(uuh(ixp,jy ,indz(2,m)  ,indexh)*(log(pint2)-log(prp(2,m))) &
                  + uuh(ixp,jy ,indz(2,m)+1,indexh)*(log(pr(2,m))-log(pint2))) &
                  / (log(pr(2,m))-log(prp(2,m)))
            v(2,m)=(vvh(ixp,jy ,indz(2,m)  ,indexh)*(log(pint2)-log(prp(2,m))) &
                  + vvh(ixp,jy ,indz(2,m)+1,indexh)*(log(pr(2,m))-log(pint2))) &
                  / (log(pr(2,m))-log(prp(2,m)))
            u(3,m)=(uuh(ix ,jyp,indz(3,m)  ,indexh)*(log(pint3)-log(prp(3,m))) &
                  + uuh(ix ,jyp,indz(3,m)+1,indexh)*(log(pr(3,m))-log(pint3))) &
                  / (log(pr(3,m))-log(prp(3,m)))
            v(3,m)=(vvh(ix ,jyp,indz(3,m)  ,indexh)*(log(pint3)-log(prp(3,m))) &
                  + vvh(ix ,jyp,indz(3,m)+1,indexh)*(log(pr(3,m))-log(pint3))) &
                  / (log(pr(3,m))-log(prp(3,m)))
            u(4,m)=(uuh(ixp,jyp,indz(4,m)  ,indexh)*(log(pint4)-log(prp(4,m))) &
                  + uuh(ixp,jyp,indz(4,m)+1,indexh)*(log(pr(4,m))-log(pint4))) &
                  / (log(pr(4,m))-log(prp(4,m)))
            v(4,m)=(vvh(ixp,jyp,indz(4,m)  ,indexh)*(log(pint4)-log(prp(4,m))) &
                  + vvh(ixp,jyp,indz(4,m)+1,indexh)*(log(pr(4,m))-log(pint4))) &
                  / (log(pr(4,m))-log(prp(4,m)))

          u1(m)=p1*u(1,m)+p2*u(2,m)+p3*u(3,m)+p4*u(4,m)
          v1(m)=p1*v(1,m)+p2*v(2,m)+p3*v(3,m)+p4*v(4,m)
  
        enddo

      case('lin')
      
        do m=1,2
          indexh=memind(m)
            u(1,m)=(uuh(ix ,jy ,indz(1,m)  ,indexh)*(pint1-prp(1,m)) &
                  + uuh(ix ,jy ,indz(1,m)+1,indexh)*(pr(1,m)-pint1)) &
                  / (pr(1,m)-prp(1,m))
            v(1,m)=(vvh(ix ,jy ,indz(1,m)  ,indexh)*(pint1-prp(1,m)) &
                  + vvh(ix ,jy ,indz(1,m)+1,indexh)*(pr(1,m)-pint1)) &
                  / (pr(1,m)-prp(1,m))  
            u(2,m)=(uuh(ixp,jy ,indz(2,m)  ,indexh)*(pint2-prp(2,m)) &
                  + uuh(ixp,jy ,indz(2,m)+1,indexh)*(pr(2,m)-pint2)) &
                  / (pr(2,m)-prp(2,m))
            v(2,m)=(vvh(ixp,jy ,indz(2,m)  ,indexh)*(pint2-prp(2,m)) &
                  + vvh(ixp,jy ,indz(2,m)+1,indexh)*(pr(2,m)-pint2)) &
                  / (pr(2,m)-prp(2,m))
            u(3,m)=(uuh(ix ,jyp,indz(3,m)  ,indexh)*(pint3-prp(3,m)) &
                  + uuh(ix ,jyp,indz(3,m)+1,indexh)*(pr(3,m)-pint3)) &
                  / (pr(3,m)-prp(3,m))
            v(3,m)=(vvh(ix ,jyp,indz(3,m)  ,indexh)*(pint3-prp(3,m)) &
                  + vvh(ix ,jyp,indz(3,m)+1,indexh)*(pr(3,m)-pint3)) &
                  / (pr(3,m)-prp(3,m))
            u(4,m)=(uuh(ixp,jyp,indz(4,m)  ,indexh)*(pint4-prp(4,m)) &
                  + uuh(ixp,jyp,indz(4,m)+1,indexh)*(pr(4,m)-pint4)) &
                  / (pr(4,m)-prp(4,m))
            v(4,m)=(vvh(ixp,jyp,indz(4,m)  ,indexh)*(pint4-prp(4,m)) &
                  + vvh(ixp,jyp,indz(4,m)+1,indexh)*(pr(4,m)-pint4)) &
                  / (pr(4,m)-prp(4,m))

          u1(m)=p1*u(1,m)+p2*u(2,m)+p3*u(3,m)+p4*u(4,m)
          v1(m)=p1*v(1,m)+p2*v(2,m)+p3*v(3,m)+p4*v(4,m)

        enddo
      
      end select     

      endif

! Calculation of z_factor
! Estimation of temperature near the particle
      tint =                                                   &
          (tth(ix,jy,indz(1,1),1)*(log(pint)-log(prp(1,1 )))   &
        + tth(ix ,jy ,indz(1,1)+1,1)*(log(pr(1,1))-log(pint))) &
        / (log(pr(1,1))-log(prp(1,1)))
!      if (jp==12929) print *,'tint simple ',tint
      select case (diftype)      
      case (1)          ! diffusion in z
        z_factor = ga/(r_air*tint)
      case (2)          ! diffusion in theta
        dTdLogp = (tth(ix,jy,indz(1,1),1)-tth(ix,jy,indz(1,1)+1,1))/ &
                  (log(pr(1,1))-log(prp(1,1)))
        z_factor = abs((pint/p0)**kappa / (kappa*tint - dTdLogp))
        z_factor = min(z_factor,0.5)   ! bound z_factor to 50 times the 
      case default                     ! tropospheric value
        z_factor = 0.
      end select
 
! Accurate calculation of the temperature if needed
 
      if (AccurateTemp) then
      
      select case(vert_interpol)
      
      case('log')
            
        do m=1,2
          indexh=memind(m)
            
            tp(1,m)=(tth(ix ,jy ,indz(1,m)  ,indexh)*(log(pint1)-log(prp(1,m))) &
                  + tth(ix ,jy ,indz(1,m)+1,indexh)*(log(pr(1,m))-log(pint1))) &
                  / (log(pr(1,m))-log(prp(1,m))) 
            tp(2,m)=(tth(ixp,jy ,indz(2,m)  ,indexh)*(log(pint2)-log(prp(2,m))) &
                  + tth(ixp,jy ,indz(2,m)+1,indexh)*(log(pr(2,m))-log(pint2))) &
                  / (log(pr(2,m))-log(prp(2,m)))
            tp(3,m)=(tth(ix ,jyp,indz(3,m)  ,indexh)*(log(pint3)-log(prp(3,m))) &
                  + tth(ix ,jyp,indz(3,m)+1,indexh)*(log(pr(3,m))-log(pint3))) &
                  / (log(pr(3,m))-log(prp(3,m)))
            tp(4,m)=(tth(ixp,jyp,indz(4,m)  ,indexh)*(log(pint4)-log(prp(4,m))) &
                  + tth(ixp,jyp,indz(4,m)+1,indexh)*(log(pr(4,m))-log(pint4))) &
                  / (log(pr(4,m))-log(prp(4,m)))
          
          tp1(m)=p1*tp(1,m)+p2*tp(2,m)+p3*tp(3,m)+p4*tp(4,m)
          
        enddo

      case('lin')
      
        do m=1,2
          indexh=memind(m)
            
            tp(1,m)=(tth(ix ,jy ,indz(1,m)  ,indexh)*(pint1-prp(1,m))  &
                  + tth(ix ,jy ,indz(1,m)+1,indexh)*(pr(1,m)-pint1))  &
                  / (pr(1,m)-prp(1,m))
            tp(2,m)=(tth(ixp,jy ,indz(2,m)  ,indexh)*(pint2-prp(2,m))  &
                  + tth(ixp,jy ,indz(2,m)+1,indexh)*(pr(2,m)-pint2))  &
                  / (pr(2,m)-prp(2,m)) 
            tp(3,m)=(tth(ix ,jyp,indz(3,m)  ,indexh)*(pint3-prp(3,m))  &
                  + tth(ix ,jyp,indz(3,m)+1,indexh)*(pr(3,m)-pint3))  &
                  / (pr(3,m)-prp(3,m)) 
            tp(4,m)=(tth(ixp,jyp,indz(4,m)  ,indexh)*(pint4-prp(4,m))  &
                  + tth(ixp,jyp,indz(4,m)+1,indexh)*(pr(4,m)-pint4))  &
                  / (pr(4,m)-prp(4,m))      
          
          tp1(m)=p1*tp(1,m)+p2*tp(2,m)+p3*tp(3,m)+p4*tp(4,m)
          
        enddo
      
      end select
      tint=(tp1(1)*dt2+tp1(2)*dt1)*dtt
!      if(jp==1) then 
!        print *,'T ',(tth(ix ,jy ,indz(1,m),memind(m)),tth(ixp,jy ,indz(2,m),memind(m)),& 
!          tth(ix,jyp,indz(3,m),memind(m)),tth(ixp,jyp,indz(4,m),memind(m)),m=1,1)
!        print *,'T ',(tth(ix ,jy ,indz(1,m)+1,memind(m)),tth(ixp,jy ,indz(2,m)+1,memind(m)),& 
!          tth(ix,jyp,indz(3,m)+1,memind(m)),tth(ixp,jyp,indz(4,m)+1,memind(m)),m=1,1)
!        print *, 'P ',pint1,pint2,pint3,pint4
!        print *, (pr(i,1),i=1,4)
!        print *, (prp(i,1),i=1,4)
!        print *,'TP ',(tp(i,1),i=1,4)
!        print *,tp1
!        print *,p1,p2,p3,p4
!        print *,tint
!      endif 
      endif

      if(debug_out) &
        print "('interpol>',i3,' P ',3f7.0,' T ',3f7.2,' TH ',3f7.2)", &
           indz(1,1),pint,pr(1,1),prp(1,1), &
           tint,tth(ix,jy,indz(1,1),memind(1)),tth(ix,jy,indz(1,1)+1,memind(1)), &
           tint*(p0/pint)**kappa,tth(ix,jy,indz(1,1),memind(1))*(p0/pr(1,1))**kappa, &
                               tth(ix,jy,indz(1,1)+1,memind(1))*(p0/prp(1,1))**kappa

! Localisation in w levels
      
      indz(1,1) = locw2(indz(1,1),psl(1,1))
      do i=2,4
        indz(i,1)=locw2(indz(1,1),psl(i,1))
      enddo
      do i=1,4
        indz(i,2)=locw2(indz(1,1),psl(i,2))
      enddo
 
      do i=1,4
        pr (i,1)=akm(indz(i,1)  )+bkm(indz(i,1)  )*psl(i,1)
        prp(i,1)=akm(indz(i,1)+1)+bkm(indz(i,1)+1)*psl(i,1)
        pr (i,2)=akm(indz(i,2)  )+bkm(indz(i,2)  )*psl(i,2)
        prp(i,2)=akm(indz(i,2)+1)+bkm(indz(i,2)+1)*psl(i,2)
      enddo

!     Patch to avoid extrapolation in the boundary layer near orography
!     and at the top of the atmosphere      
      pint1=min(pint,min(pr(1,1),pr(1,2)))
      pint2=min(pint,min(pr(2,1),pr(2,2)))
      pint3=min(pint,min(pr(3,1),pr(3,2)))
      pint4=min(pint,min(pr(4,1),pr(4,2)))
      if(maxval(indz)==nwz-1) pint=max(pint,maxval(prp)) ! partial patch

      select case(vert_interpol)
      
      case('log') 
           
        do m=1,2
          indexh=memind(m)
        
            w(1,m)=(wwh(ix ,jy ,indz(1,m)  ,indexh)*(log(pint1)-log(prp(1,m))) &
                  + wwh(ix ,jy ,indz(1,m)+1,indexh)*(log(pr(1,m))-log(pint1))) &
                  / (log(pr(1,m))-log(prp(1,m)))
            w(2,m)=(wwh(ixp,jy ,indz(2,m)  ,indexh)*(log(pint2)-log(prp(2,m))) &
                  + wwh(ixp,jy ,indz(2,m)+1,indexh)*(log(pr(2,m))-log(pint2))) &
                  / (log(pr(2,m))-log(prp(2,m)))
            w(3,m)=(wwh(ix ,jyp,indz(3,m)  ,indexh)*(log(pint3)-log(prp(3,m))) &
                  + wwh(ix ,jyp,indz(3,m)+1,indexh)*(log(pr(3,m))-log(pint3))) &
                  / (log(pr(3,m))-log(prp(3,m)))
            w(4,m)=(wwh(ixp,jyp,indz(4,m)  ,indexh)*(log(pint4)-log(prp(4,m))) &
                  + wwh(ixp,jyp,indz(4,m)+1,indexh)*(log(pr(4,m))-log(pint4))) &
                  / (log(pr(4,m))-log(prp(4,m)))

          w1(m)=p1*w(1,m)+p2*w(2,m)+p3*w(3,m)+p4*w(4,m)

        enddo
      
      case('lin')
        
        do m=1,2
          indexh=memind(m)
        
            w(1,m)=(wwh(ix ,jy ,indz(1,m)  ,indexh)*(pint1-prp(1,m))  &
                  + wwh(ix ,jy ,indz(1,m)+1,indexh)*(pr(1,m)-pint1))  &
                  / (pr(1,m)-prp(1,m))
            w(2,m)=(wwh(ixp,jy ,indz(2,m)  ,indexh)*(pint2-prp(2,m))  &
                  + wwh(ixp,jy ,indz(2,m)+1,indexh)*(pr(2,m)-pint2))  &
                  / (pr(2,m)-prp(2,m))
            w(3,m)=(wwh(ix ,jyp,indz(3,m)  ,indexh)*(pint3-prp(3,m))  &
                  + wwh(ix ,jyp,indz(3,m)+1,indexh)*(pr(3,m)-pint3))  &
                  / (pr(3,m)-prp(3,m))
            w(4,m)=(wwh(ixp,jyp,indz(4,m)  ,indexh)*(pint4-prp(4,m))  &
                  + wwh(ixp,jyp,indz(4,m)+1,indexh)*(pr(4,m)-pint4))  &
                  / (pr(4,m)-prp(4,m))

          w1(m)=p1*w(1,m)+p2*w(2,m)+p3*w(3,m)+p4*w(4,m)

        enddo

      end select

!************************************
! 3.) Temporal interpolation (linear)
!************************************

      dxdt=(u1(1)*dt2+u1(2)*dt1)*dtt
      dydt=(v1(1)*dt2+v1(2)*dt1)*dtt
      dzdt=(w1(1)*dt2+w1(2)*dt1)*dtt
      !---- Temporary test
      !write(*,'(a,3g12.3)') 'interpol> u,v,w ',dxdt,dydt,dzdt 
      !----
      
      return

!************************************
      contains
!************************************      
    
      function locuv(ib1,iu1,psu)
      integer, intent(in) :: ib1,iu1
      real, intent(in) :: psu
      integer :: locuv,ib,iu,im
      real :: pm
      ib=ib1 ; iu=iu1
      do while(iu-ib>1)
        im = (iu+ib)/2
        pm = akz(im) + bkz(im)*psu
        if( pm >= pint ) then
          ib=im
        else
          iu=im
        endif
      enddo
      locuv = ib
      end function locuv
      
      function locuv2(ib1,psu)
      integer, intent(in) :: ib1
      real, intent(in) :: psu
      integer :: locuv2,ib
      real :: pm
      ib=ib1
      pm = akz(ib) + bkz(ib)*psu
      if (pm < pint) then
        do while(ib > 1)
          ib = ib-1
          pm = akz(ib) + bkz(ib)*psu
          if (pm >= pint) exit
        enddo
      else
        do while(ib < nuvz-1)
          pm = akz(ib+1) + bkz(ib+1)*psu
          if (pm < pint) exit
          ib = ib+1
        enddo
      endif
      locuv2 = ib
      end function locuv2 

      function locw(ib1,iu1,psu)
      integer, intent(in) :: ib1, iu1
      real, intent(in) :: psu
      integer :: locw,ib,iu,im
      real :: pm
      ib=ib1 ; iu=iu1
      do while(iu-ib>1)
        im = (iu+ib)/2
        pm = akm(im) + bkm(im)*psu
        if( pm >= pint ) then
          ib=im
        else
          iu=im
        endif
      enddo
      locw = ib
      end function locw
      
      function locw2(ib1,psu)
      integer, intent(in) :: ib1
      real, intent(in) :: psu
      integer :: locw2,ib
      real :: pm
      ib=ib1
      pm = akm(ib) + bkm(ib)*psu
      if (pm < pint) then
        do while(ib > 1)
          ib = ib-1
          pm = akm(ib) + bkm(ib)*psu
          if (pm >= pint) exit
        enddo
      else
        do while(ib < nwz-1)
          pm = akm(ib+1) + bkm(ib+1)*psu
          if (pm < pint) exit
          ib = ib+1
        enddo
      endif
      locw2 = ib
      end function locw2 

      end subroutine interpol_windB

!
!=====|==1=========2=========3=========4=========5=========6=========7==
!
!$Log:
