!$Id:
!
!===========================================================================
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ FLEXPART @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!=====|==1=========2=========3=========4=========5=========6=========7======
!
 subroutine readcommandB(error)

!***************************************************************************
!    This routine reads the user specifications for the current model run
!
!    Replaces readcommand.f by using namelist, providing more flexibility
!    for extensions
!
!    Author: B. Legras
!    1 June 2002
!
!***************************************************************************
!    Variables (not in namelists)
!
!  bdate                beginning date as Julian date 
!  edate                ending date as Julian date 
!  hhh                  hour
!  mi                   minute
!  ss                   second
!  error                .true., if error ocurred in subprogram, else .false.
!  ideltas [s]          modelling period
!  method               method used to compute the particle pseudovelocities  
 
!  unitcommand          unit connected to file COMMAND 
!***************************************************************************

 use commons
 use isentrop_h
 use ecmwf_diab
 use ecmwf_inct
 use mass_iso
 
 implicit none
 character(len=72):: line
 logical error
 double precision juldate
 
 namelist /COMMAND/     &
   ldirect,             & ! direction of integration
   ibdate, ibtime,      & ! beginnning date and time (YYYYMMDD, HHMISS)
   iedate, ietime,      & ! ending date and time (YYYYMMDD, HHMISS)
   loutstep,            & ! time interval of concentration output [s]
   loutaver,            & ! concentration output is an average over 
                          ! loutaver [s]
   loutsample,          & ! average is computed from samples taken every 
                          ! loutsample [s]
   itsplit,             & ! time constant for particle splitting [s]
   lsynctime,           & ! synchronisation time interval for all particles[s]
   loffset,             & ! offset time for the first output [s]
                          ! (allows to start at any time)
   loffset2,            & ! second offset of the first output when out of
                          ! sequence with the following ones [s]
   ctl,                 & ! factor by which time step must be smaller than *
                          ! Lagrangian time scale
   ifine,               & ! reduction factor for vertical wind time step
   iout,                & ! 1 for conc.output, 2 for mixing ratio output, 3 both 
   ipout,               & ! 0 no part. dump, 1 every output time, 3 only at end
   lsubgrid,            & ! switch to turn on/off subgrid topography param.
   lagespectra,         & ! switch to turn on (1)/off (2) calc. of age spectra
   ipin,                & ! 1 continue simul. with dumped particle data, 2 no 
   iflux,               & ! switch to turn on (1)/off (2) flux calculations
   lconvection,         & ! switch to turn on (1)/off (0)convection param.
   diffus,              & ! vertical diffusivity  [unit depending on diftype]
   diftype,             & ! type of vertical diffusivity, 1:z, 2:theta
   verdiff,             & ! vertical diffusion
   hordiff,             & ! horizontal diffusion
   release_plan,        & ! type of release method, 'o3sonde', 'MOZAIC'
   vert_interpol,       & ! type of vertical interpolation in p, 'log', 'lin'
   restart,hrstart,     & ! restart run, hour labeling the restart file
   correct_vertwind,    & ! correcting vertical wind
   delay_switch_diff_off,& ! diffusion is switched off after this delay (in s)
   diabatic_w,          & ! diabatic velocities are used
   ecmwf_diabatic_w,    & ! diabatic velocities from ecmwf archive
   clear_sky,           & ! clear sky version of diabatic velocities
   cloud_sky,           & ! cloud sky version of diabatic velocities (no latent heat)
   ecmwf_inct_w,        & ! temperature increment from ecmwf
   z_motion,            & ! motion is in z coordinate
   isentropic_motion,   & ! isentropic motion
   theta_bounds,        & ! theta excursion is bounded
   upper_theta_level,   & ! upper level on which theta is calculated  
   lower_theta_level,   & ! lower level on which theta is calculated
   debug_out,           & ! test prints to check diabatic versus mass velocities
   perpetual,           & ! perpetual run
   iso_mass,            & ! data on isentropic levels, mass equilibrated
   mass_diabat,         & ! use heating rates on isentropic levels
   mass_isent,          & ! isentropic motion from data on isentropic levels
   mass_correction,     & ! heating rate correction to preserve mass conservation in the stratosphere
   mean_diab_output       ! Output of the mass averaged heating rate 
   
!Open the command file and read user options
!-------------------------------------------

 error=.false.
 open(unitcommand,file=path(1)(1:len(1))//'COMMAND',status='old',err=999)

! Check the format of the COMMAND file and call old readcommand
! if new heading with COMMAND in the first line is not detected
!--------------------------------------------------------------

 read (unitcommand,'(a)') line
 if (index(line,'COMMAND') == 0 ) then
   close(unitcommand)
   command_old=.true.
!   call readcommand(error)
   print *,' OLD COMMAND FORMAT NOW OBSOLETE'
   print *,' USE NEW FORMAT'
   error = .true.
   return
 endif

! Default values for some parameters
 ldirect = -1
 loutstep = 43200
 loutaver = 3600
 loutsample = 1800
 itsplit = 999999999
 lsynctime = 900
 loffset = 0
 loffset2 = 0
 ctl = 5.
 ifine = 4
 iout = 2
 ipout = 1
 lsubgrid = 2
 lagespectra = 2
 ipin = 2
 iflux = 2
 lconvection = 0
 diftype = 1
 diffus = 0.
 verdiff=.true.
 hordiff=.false.
 release_plan='o3sonde'
 vert_interpol='log'
 restart=.false.
 hrstart=0
 correct_vertwind = .true.
 delay_switch_diff_off=0
 switch_diff_off=.false.
 diabatic_w=.false.
 ecmwf_diabatic_w=.false.
 clear_sky=.false.
 cloud_sky=.false.
 ecmwf_inct_w=.false.
 z_motion=.true.
 isentropic_motion=.false.
 theta_bounds=.true.
 lower_theta_level=2
 upper_theta_level=1000
 debug_out=.false.
 perpetual=.false.
 iso_mass=.false.
 mass_diabat=.false.
 mass_isent=.false.
 mass_correction=.false.
 mean_diab_output=.false.
 
 read(unitcommand,NML=COMMAND)
 
 ifine=max(ifine,1)

! Determine how Markov chain is formulated (for w or for w/sigw)
!---------------------------------------------------------------
 if (ctl.ge.0.1) then
   turbswitch=.true.
 else
   turbswitch=.false.
   ifine=1
 endif
 fine=1./float(ifine)
 ctl=1./ctl

! Check input dates
!------------------
 error = (check_input_date() == 1).or.error
      
! Determine kind of dispersion method
!------------------------------------
 if (ctl > 0.) then    
   method=1
   mintime=minstep
 else
   method=0
   mintime=lsynctime
 endif


! Checks 
!    output options
!    whether synchronisation interval is sufficiently short
!    consist of gridded concentration output
!----------------------------------------------------------
 error = (check_output_options() == 1).or.error
 error = (check_synchro() == 1).or.error
 error = (check_consist() == 1).or.error

! Compute modeling time in seconds and beginning date in Julian date
!-------------------------------------------------------------------
 if (ldirect.eq.1) then
   bdate=juldate(ibdate,ibtime)
   edate=juldate(iedate,ietime)
   ideltas=int((edate-bdate)*86400.)
 else if (ldirect.eq.-1) then
   loutaver=-1*loutaver
   loutstep=-1*loutstep
   loutsample=-1*loutsample
   lsynctime=-1*lsynctime
   bdate=juldate(iedate,ietime)
   edate=juldate(ibdate,ibtime)
   ideltas=int((edate-bdate)*86400.)
 else
     write(*,*) ' #### FLEXPART MODEL ERROR! DIRECTION IN      #### '
     write(*,*) ' #### FILE "COMMAND" MUST BE EITHER -1 OR 1.  #### '
     error=.true.
     return
 endif
 print *,'readcommand > direction ',ldirect
 print *,'readcommand > ',ibdate, ibtime
 print *,'readcommand > ',iedate, ietime
 print *,'readcommand > diffusion ',diftype,diffus
 print *,'readcommand > ',release_plan
 print *,'readcommand > ',vert_interpol
 print *,'readcommand > restart ',restart, hrstart
 print *,'readcommand > z_motion ',z_motion
 print *,'readcommand > isentropic_motion ',isentropic_motion
 print *,'readcommand > diabatic_w ',diabatic_w
 print *,'readcommand > ecmwf_diabatic_w ',ecmwf_diabatic_w
 print *,'readcommand > clear_sky ',clear_sky
 print *,'readcommand > cloud_sky ',cloud_sky
 print *,'readcommand > ecmwf_inct_w ',ecmwf_inct_w
 print *,'readcommand > theta_bounds ',theta_bounds,lower_theta_level,&
                                                   upper_theta_level
 print *,'readcommand > debug_out ',debug_out
 print *,'readcommand > iso_mass mass_diabat ',iso_mass,mass_diabat
 print *,'readcommand > mass_correction mean_diab_output ',mass_correction,mean_diab_output
 
! Conversion of format HHHMISS to seconds
!----------------------------------------
! hhh=ietime/10000
! mi=(ietime-10000*hhh)/100
! ss=ietime-10000*hhh-100*mi

! reset loffset2 if restart run
! so that one does not need to rest it manually in the auto script 
 if(restart) loffset2=0

 return    

 999  write(*,*) ' #### FLEXPART MODEL ERROR! FILE "COMMAND"    #### ' 
      write(*,*) ' #### CANNOT BE OPENED IN THE DIRECTORY       #### '
      write(*,*) ' #### xxx/flexpart/options                    #### '
      print *,path(1)(1:len(1))//'COMMAND'
      error=.true.

 return
      
 contains
      
      function check_input_date ()
      integer :: check_input_date
      check_input_date=0
      if (iedate.lt.ibdate) then
        write(*,*) ' #### FLEXPART MODEL ERROR! BEGINNING DATE    #### ' 
        write(*,*) ' #### IS LARGER THAN ENDING DATE. CHANGE      #### '
        write(*,*) ' #### EITHER POINT 2 OR POINT 3 IN FILE       #### '
        write(*,*) ' #### "COMMAND".                              #### '
        check_input_date=1
      else if (iedate.eq.ibdate) then
        if (ietime.lt.ibtime) then
          write(*,*) ' #### FLEXPART MODEL ERROR! BEGINNING TIME    #### ' 
          write(*,*) ' #### IS LARGER THAN ENDING TIME. CHANGE      #### '
          write(*,*) ' #### EITHER POINT 2 OR POINT 3 IN FILE       #### '
          write(*,*) ' #### "COMMAND".                              #### '
          check_input_date=1
        endif
      endif
      end function check_input_date

      function check_output_options()
      integer :: check_output_options
      check_output_options = 0
      if ((iout.ne.1).and.(iout.ne.2).and.(iout.ne.3)) then
        write(*,*) ' #### FLEXPART MODEL ERROR! NEITHER CONCENTRAT#### ' 
        write(*,*) ' #### ION NOR MIXING RATIO OUTPUT SELECTED.   #### '
        check_output_options = 1
      endif
      if ((ipout.ne.0).and.(ipout.ne.1).and.(ipout.ne.2)) then
        write(*,*) ' #### FLEXPART MODEL ERROR! FILE COMMAND:     #### ' 
        write(*,*) ' #### IPOUT MUST BE 1, 2 OR 3!                #### '
        check_output_options = 1
      endif
      if(lsubgrid.ne.1) then
        write(*,*) '             ----------------               '
        write(*,*) ' INFORMATION: SUBGRIDSCALE TERRAIN EFFECT IS'
        write(*,*) ' NOT PARAMETERISED DURING THIS SIMULATION...'
        write(*,*) '             ----------------               '
      endif
      end function check_output_options
      
      function check_synchro()
      integer :: check_synchro
      check_synchro = 0      
      if (lsynctime.gt.(idiffnorm/2)) then
        write(*,*) ' #### FLEXPART MODEL ERROR! SYNCHRONISATION   #### ' 
        write(*,*) ' #### TIME IS TOO LONG. MAKE IT SHORTER.      #### '
        check_synchro = 1
      endif
      end function check_synchro
  
      function check_consist()
      integer :: check_consist
      check_consist = 0
      if (loutaver.eq.0) then
        write(*,*) ' #### FLEXPART MODEL ERROR! TIME AVERAGE OF   #### ' 
        write(*,*) ' #### CONCENTRATION FIELD OUTPUT MUST NOT BE  #### '
        write(*,*) ' #### ZERO.                                   #### '
        write(*,*) ' #### CHANGE INPUT IN FILE COMMAND.           #### '
        check_consist = 1
      endif
      if (loutaver.gt.loutstep) then
        write(*,*) ' #### FLEXPART MODEL ERROR! TIME AVERAGE OF   #### ' 
        write(*,*) ' #### CONCENTRATION FIELD OUTPUT MUST NOT BE  #### '
        write(*,*) ' #### GREATER THAN INTERVAL OF OUTPUT.        #### '
        write(*,*) ' #### CHANGE INPUT IN FILE COMMAND.           #### '
        check_consist = 1
      endif
      if (loutsample.gt.loutaver) then
        write(*,*) ' #### FLEXPART MODEL ERROR! SAMPLING TIME OF  #### ' 
        write(*,*) ' #### CONCENTRATION FIELD OUTPUT MUST NOT BE  #### '
        write(*,*) ' #### GREATER THAN TIME AVERAGE OF OUTPUT.    #### '
        write(*,*) ' #### CHANGE INPUT IN FILE COMMAND.           #### '
        check_consist = 1
      endif
      if (mod(loutaver,lsynctime).ne.0) then
        write(*,*) ' #### FLEXPART MODEL ERROR! AVERAGING TIME OF #### ' 
        write(*,*) ' #### CONCENTRATION FIELD MUST BE A MULTIPLE  #### '
        write(*,*) ' #### OF THE SYNCHRONISATION INTERVAL         #### '
        check_consist = 1
      endif
      if (mod(loutstep,lsynctime).ne.0) then
        write(*,*) ' #### FLEXPART MODEL ERROR! INTERVAL BETWEEN  #### ' 
        write(*,*) ' #### CONCENTRATION FIELDS MUST BE A MULTIPLE #### '
        write(*,*) ' #### OF THE SYNCHRONISATION INTERVAL         #### '
        check_consist = 1
      endif
      if (mod(loutsample,lsynctime).ne.0) then
        write(*,*) ' #### FLEXPART MODEL ERROR! SAMPLING TIME OF  #### ' 
        write(*,*) ' #### CONCENTRATION FIELD MUST BE A MULTIPLE  #### '
        write(*,*) ' #### OF THE SYNCHRONISATION INTERVAL         #### '
        check_consist = 1
      endif
      if (itsplit.lt.loutaver) then
        write(*,*) ' #### FLEXPART MODEL ERROR! SPLITTING TIME FOR#### ' 
        write(*,*) ' #### PARTICLES IS TOO SHORT. PLEASE INCREASE #### '
        write(*,*) ' #### SPLITTING TIME CONSTANT.                #### '
        check_consist = 1
      endif
      if ((lconvection.ne.0).and.(lconvection.ne.1)) then
        write(*,*) ' #### FLEXPART MODEL ERROR! LCONVECTION MUST  #### ' 
        write(*,*) ' #### BE SET TO EITHER 1 OR 0 IN FILE COMMAND.#### ' 
        check_consist = 1
      endif
      end function check_consist
      
 end subroutine readcommandB

!
!=====|==1=========2=========3=========4=========5=========6=========7==
!
!$Log:
 
