!$Id:
!
!=======================================================================
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ FLEXPART @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!=====|==1=========2=========3=========4=========5=========6=========7==
!
      subroutine readwind(indj,n)
!**********************************************************************
!                                                                     * 
!             TRAJECTORY MODEL SUBROUTINE READWIND                    *
!                                                                     *
!**********************************************************************
!                                                                     * 
!             AUTHOR:      G. WOTAWA                                  *
!             DATE:        1997-08-05                                 *
!             LAST UPDATE: 2000-10-17, Andreas Stohl                  *
!                                                                     * 
!**********************************************************************
!  Changes, Bernd C. Krueger, Feb. 2001:
!        Variables tth and qvh (on eta coordinates) in common block
!           B. Legras, Apr. 2002:
!        Two times for uuh, vvh, wwh, in common
!        select case + loop inside
!**********************************************************************
!                                                                     *
! DESCRIPTION:                                                        *
!                                                                     *
! READING OF ECMWF METEOROLOGICAL FIELDS FROM INPUT DATA FILES. THE   *
! INPUT DATA FILES ARE EXPECTED TO BE AVAILABLE IN GRIB CODE          *
!                                                                     *
! INPUT:                                                              *
! indj               indicates number of the wind field to be read in *
! n                  temporal index for meteorological fields (1 to 3)*
!                                                                     *
! IMPORTANT VARIABLES FROM COMMON BLOCK:                              *
!                                                                     *
! wfname             File name of data to be read in                  *
! nx,ny,nuvz,nwz     expected field dimensions                        *
! nlev_ec            number of vertical levels ecmwf model            *
! uu,vv,ww           wind fields  m/s and Pa/s                        *
! tt,qv              temperature and specific humidity                *
! ps                 surface pressure                                 *
!                                                                     *
!**********************************************************************
!
      use commons
      use isentrop_h, only: isentropic_motion,theta_col,theta_inv_col
      use isentrop_m  
      use ecmwf_diab, only: diabatic_w,mass_correction
      implicit none

      integer, intent(in):: indj,n
      integer i,j,k,levdiff2,ifield,iumax,iwmax,lunit

! VARIABLES AND ARRAYS NEEDED FOR GRIB DECODING

! dimension of isec2 at least (22+n), where n is the number of parallels or
! meridians in a quasi-regular (reduced) Gaussian or lat/long grid

! dimension of zsec2 at least (10+nn), where nn is the number of vertical
! coordinate parameters

      integer :: isec0(2),isec1(56),isec3(2)
      integer :: isec4(64),ilen,ierr,iword
      integer, allocatable :: inbuff(:), isec2(:)
      real :: zsec3(2)
      real, allocatable :: zsec4(:), zsec2(:)
      real :: xaux,yaux,xaux0,yaux0
!     real, allocatable :: ewss(:,:),nsss(:,:)

      character(len=1):: yoper
      logical :: hflswitch,strswitch
      
      integer :: OMP_GET_NUM_PROCS

      data yoper/'D'/

      hflswitch=.false.
      strswitch=.false.
      levdiff2=nlev_ec-nwz+1
      iumax=0
      iwmax=0
      allocate (zsec4(jpunp),inbuff(jpack))
      allocate (isec2(22+nx+ny),zsec2(60+nuvz+nwz))
!     allocate (ewss(0:nx-1,0:ny-1),nsss(0:nx-1,0:ny-1))
      
!
! OPENING OF DATA FILE (GRIB CODE)
!
      call pbopen(lunit,path(3)(1:len(3))//trim(wfname(indj)),'r',ierr)
      if(ierr < 0) goto 999
!     print *,'readwind> file ',wfname(indj)
 
      ifield=0
      
10    ifield=ifield+1
!
! GET NEXT FIELDS
!
      call pbgrib(lunit,inbuff,jpack,ilen,ierr)
      if(ierr.eq.-1) goto 50    ! EOF DETECTED
      if(ierr.lt.-1) goto 888   ! ERROR DETECTED

      ierr=1

! Check whether we are on a little endian or on a big endian computer
!********************************************************************
! no longer needed with new emoslib
!
!     if (inbuff(1).eq.1112101447) then         ! little endian, swap bytes
!!      if(kind(1)==4) then
!!        iswap=1+ilen/4
!!        call swap32(inbuff,iswap)
!!      else
!!        iswap=1+ilen/8
!!        call swap64(inbuff,iswap)
!!      endif          
!     else if (inbuff(1).ne.1196575042) then    ! big endian
!       stop 'subroutine gridcheck: corrupt GRIB data'
!     endif

      call gribex(isec0,isec1,isec2,zsec2,isec3,zsec3,isec4,  &
                  zsec4,jpunp,inbuff,jpack,iword,yoper,ierr)
      if (ierr.ne.0) goto 888   ! ERROR DETECTED

      if(ifield.eq.1) then

! CHECK GRID SPECIFICATIONS

        if(isec2(2).ne.nxfield) stop 'READWIND: NX NOT CONSISTENT'
        if(isec2(3).ne.ny) stop 'READWIND: NY NOT CONSISTENT'
        if(isec2(12)/2-1.ne.nlev_ec) &
          stop 'READWIND: VERTICAL DISCRETIZATION NOT CONSISTENT'
        xaux=float(isec2(5))/1000.
        yaux=float(isec2(7))/1000.
        xaux0=xlon0
        yaux0=ylat0
        if(xaux<0.) xaux=xaux+360.
!       if(yaux<0.) yaux=yaux+360.           useless & confusing 
        if(xaux0<0.) xaux0=xaux0+360.
!       if(yaux0<0.) yaux0=yaux0+360.        useless & confusing
        if(abs(xaux-xaux0)>1.e-3) then
          print *, xaux, xaux0
          stop 'READWIND: LOWER LEFT LONGITUDE NOT CONSISTENT'
        endif
        if(abs(yaux-yaux0)>1.e-3) then
          print *,yaux,yaux0
          stop 'READWIND: LOWER LEFT LATITUDE NOT CONSISTENT'
        endif
      endif

      k=isec1(8)
      field_identifier: select case (isec1(6))
! 3D fields
      case (130)              !! TEMPERATUR
        do j=0,ny-1 ; do i=0,nxfield-1
          tth(i,j,nlev_ec-k+2,n) = zsec4(nxfield*(ny-j-1)+i+1)
        enddo       ; enddo
      case (131)               !! U VELOCIT  
        do j=0,ny-1 ; do i=0,nxfield-1
          uuh(i,j,nlev_ec-k+2,n) = zsec4(nxfield*(ny-j-1)+i+1)
        enddo       ; enddo
        iumax=max(iumax,nlev_ec-k+1)
      case (132)              !! V VELOCITY   
        do j=0,ny-1 ; do i=0,nxfield-1
          vvh(i,j,nlev_ec-k+2,n) = zsec4(nxfield*(ny-j-1)+i+1)
        enddo       ; enddo
      case (133)              !! SPEC. HUMIDITY 
        if(TTLactiv) then
          do j=0,ny-1 ; do i=0,nxfield-1
            qvh(i,j,nlev_ec-k+2,n) = max(zsec4(nxfield*(ny-j-1)+i+1),0.)
          enddo       ; enddo
        endif 
      case (135)              !! W VELOCITY
        if(z_motion) then
          do j=0,ny-1 ; do i=0,nxfield-1
             wwh(i,j,nlev_ec-k+1,n) = zsec4(nxfield*(ny-j-1)+i+1)
          enddo       ; enddo 
          iwmax=max(iwmax,nlev_ec-k+1)
        endif
! surface fields
      case (134)              !! SURF. PRESS.
        do j=0,ny-1 ; do i=0,nxfield-1
           ps(i,j,1,n) = zsec4(nxfield*(ny-j-1)+i+1)
        enddo       ; enddo            
      case (141)              !! SNOW DEPTH
!        do j=0,ny-1 ; do i=0,nxfield-1
!           sd(i,j,1,n) = zsec4(nxfield*(ny-j-1)+i+1)
!        enddo       ; enddo
!      case (151)              !! SEA LEVEL PRESS.
!        do j=0,ny-1 ; do i=0,nxfield-1
!           msl(i,j,1,n) = zsec4(nxfield*(ny-j-1)+i+1)
!        enddo       ; enddo
!     case (164)              !! CLOUD COVER
!        do j=0,ny-1 ; do i=0,nxfield-1
!           tcc(i,j,1,n) = zsec4(nxfield*(ny-j-1)+i+1)
!        enddo       ;  enddo
      case (165)              !! 10 M U VELOCITY
        do j=0,ny-1 ; do i=0,nxfield-1
           u10(i,j,1,n) = zsec4(nxfield*(ny-j-1)+i+1)
        enddo       ;  enddo
      case (166)              !! 10 M V VELOCITY
        do j=0,ny-1 ; do i=0,nxfield-1
           v10(i,j,1,n) = zsec4(nxfield*(ny-j-1)+i+1)
        enddo       ;  enddo
      case (167)              !! 2 M TEMPERATURE
        do j=0,ny-1 ; do i=0,nxfield-1
           tt2(i,j,1,n) = zsec4(nxfield*(ny-j-1)+i+1)
        enddo       ;  enddo
!      case (168)              !! 2 M DEW POINT
!        do j=0,ny-1 ; do i=0,nxfield-1
!           td2(i,j,1,n) = zsec4(nxfield*(ny-j-1)+i+1)
!        enddo       ;  enddo
!      case (142)              !! LARGE SCALE PREC.
!        do j=0,ny-1 ; do i=0,nxfield-1
!           lsprec(i,j,1,n) = max(zsec4(nxfield*(ny-j-1)+i+1),0.)
!        enddo       ;  enddo
!      case (143)              !! CONVECTIVE PREC.
!        do j=0,ny-1 ; do i=0,nxfield-1
!           convecprec(i,j,1,n) = max(zsec4(nxfield*(ny-j-1)+i+1),0.)
!        enddo       ; enddo
!      case (146)              !! SENS HEAT FLUX
!        do j=0,ny-1 ; do i=0,nxfield-1
!           sshf(i,j,1,n) = zsec4(nxfield*(ny-j-1)+i+1)
!        enddo       ; enddo 
!        if((maxval(sshf(0:nxfield-1,0:ny-1,1,n)).ne.0.).and.   &
!           (minval(sshf(0:nxfield-1,0:ny-1,1,n)).ne.0.))       &
!           hflswitch=.true.
!      case (176)              !! SOLAR RADIATION
!        do j=0,ny-1 ; do i=0,nxfield-1
!           ssr(i,j,1,n) = max(zsec4(nxfield*(ny-j-1)+i+1),0.)
!        enddo       ; enddo
!      case (180)              !! EsW SURFACE STRESS
!        do j=0,ny-1 ; do i=0,nxfield-1
!           ewss(i,j) = zsec4(nxfield*(ny-j-1)+i+1)
!        enddo       ; enddo
!        if((maxval(ewss(0:nxfield-1,0:ny-1)).ne.0.).and.       &
!         (minval(ewss(0:nxfield-1,0:ny-1)).ne.0.))             &
!           strswitch =.true.
!      case (181)              !! NS SURFACE STRESS
!        do j=0,ny-1 ; do i=0,nxfield-1
!           nsss(i,j) = zsec4(nxfield*(ny-j-1)+i+1)
!        enddo       ; enddo
!        if((maxval(nsss(0:nxfield-1,0:ny-1)).ne.0.).and.       &
!         (minval(nsss(0:nxfield-1,0:ny-1)).ne.0.))             &
!          strswitch =.true.
! orography	    
!      case (129)            !! ECMWF OROGRAPHY
!        do j=0,ny-1 ; do i=0,nxfield-1
!           oro(i,j) = zsec4(nxfield*(ny-j-1)+i+1)/ga
!        enddo       ; enddo
!      case (160)              !! STANDARD DEVIATION OF OROGRAPHY
!        do j=0,ny-1 ; do i=0,nxfield-1
!           excessoro(i,j) = zsec4(nxfield*(ny-j-1)+i+1)
!        enddo       ; enddo
!      case (172)              !! ECMWF LAND SEA MASK
!        do j=0,ny-1 ; do i=0,nxfield-1
!           lsm(i,j) = zsec4(nxfield*(ny-j-1)+i+1)
!        enddo       ; enddo
      end select field_identifier

      goto 10              !! READ NEXT LEVEL OR PARAMETER
!
! CLOSING OF INPUT DATA FILE
!
50    call pbclose(lunit,ierr)     !! FINNISHED READING / CLOSING GRIB FILE
      
!     print *,'readwind> ',uuh(186,136,24,n),vvh(186,136,24,n),wwh(186,136,24,n)

!* Defines vertical velocity on the last level to be zero or to be half
!* of the last-1, according to the type of vertical interpolation

      if(z_motion.and.(levdiff2.eq.0)) then
        iwmax=nlev_ec+1
        select case (vert_interpol)
        case ('lin')
        wwh(:,:,nwz,n)=0.                      ! linear interpol 
        case ('log')
        wwh(:,:,nwz,n)=0.5*wwh(:,:,nwz-1,n)    ! log    interpol
        case default
        stop 'ERROR 1 in readwind'
        end select
      endif


! For global fields, assign rightmost grid point the value of the
! leftmost point
!****************************************************************

      if (xglobal) then
        ps(nx-1,:,1,n)=ps(0,:,1,n)
!        ewss(nx-1,:)=ewss(0,:)
!        nsss(nx-1,:)=nsss(0,:)
!        oro(nx-1,:,1,n)=oro(0,:,1,n)
!        excessoro(nx-1,:,1,n)=escessoro(0,:,1,n)
!        lsm(nx-1,:,1,n)=lsm(0,:,1,n)
!        sd(nx-1,:,1,n)=sd(0,:,1,n)
!        msl(nx-1,:,1,n)=msl(0,:,1,n)
!        tcc(nx-1,:,1,n)=tcc(0,:,1,n)
        tt2(nx-1,:,1,n)=tt2(0,:,1,n)
!        td2(nx-1,:,1,n)=td2(0,:,1,n)
!        lsprec(nx-1,:,1,n)=lsprec(0,:,1,n)
!        convprec(nx-1,:,1,n)=convprec(0,:,1,n)
!        sshf(nx-1,:,1,n)=sshf(0,:,1,n)
!        ssr(nx-1,:,1,n)=ssr(0,:,1,n)
        u10(nx-1,:,1,n)=u10(0,:,1,n)
        u10(nx-1,:,1,n)=u10(0,:,1,n)
        uuh(nx-1,:,:,n)=uuh(0,:,:,n)
        vvh(nx-1,:,:,n)=vvh(0,:,:,n)        
        tth(nx-1,:,:,n)=tth(0,:,:,n)
        if(TTLactiv) qvh(nx-1,:,:,n)=qvh(0,:,:,n)
        if(z_motion) wwh(nx-1,:,:,n)=wwh(0,:,:,n)
      endif

! Initialize theta control (and theta, if needed)
!------------------------------------------------

      if(diabatic_w.or.isentropic_motion) then
!      	if(OMP_GET_NUM_PROCS()==1 .AND. .NOT.mass_correction) then
!       correct that as it is not defined outside a // region
!       have a variable which lets the code know the number of processors
!       declared in the PBS preamble
        if(OMP_GET_NUM_PROCS()==1) then 
          theta_col(:,:,n)=.false.
          theta_inv_col(:,:,n)=.false.
          print *,'NOCOL'
        else
!$OMP PARALLEL DO DEFAULT(SHARED) SCHEDULE(DYNAMIC) PRIVATE(i,j)
          do j=0,ny-1
             do i=0,nx-1
                call calc_col_theta(i,j,n)
             enddo
          enddo
!$OMP END PARALLEL DO
        endif
      endif

!      surfstr(:,:,1,n)=sqrt(ewss(:,:)**2+nsss(:,:)**2)

!      if ((.not.hflswitch).or.(.not.strswitch)) then
 
! CALCULATE USTAR AND SSHF USING THE PROFILE METHOD
!**************************************************

!        do i=0,nx-1
!          do j=0,ny-1
!            plev1=akz(2)+bkz(2)*ps(i,j,1,n)
!            pmean=0.5*(ps(i,j,1,n)+plev1)
!            tv=tth(i,j,2,n)*(1.+0.61*qvh(i,j,2,n))
!            fu=-r_air*tv/ga/pmean
!            hlev1=fu*(plev1-ps(i,j,1,n))   ! HEIGTH OF FIRST MODEL LAYER
!            ff10m= sqrt(u10(i,j,1,n)**2+v10(i,j,1,n)**2)
!            fflev1=sqrt(uuh(i,j,1,n)**2+vvh(i,j,1,n)**2) ! modified 
!            call pbl_profile(ps(i,j,1,n),td2(i,j,1,n),hlev1,         &
!                             tt2(i,j,1,n),tth(i,j,2,n),ff10m,fflev1, &
!                             surfstr(i,j,1,n),sshf(i,j,1,n))
!            if(sshf(i,j,1,n).gt.200.) sshf(i,j,1,n)=200.
!            if(sshf(i,j,1,n).lt.-400.) sshf(i,j,1,n)=-400.
!          enddo
!       enddo
!      endif


! Assign 10 m wind to model level at eta=1.0 to have one additional model
! level at the ground
! Specific humidity is taken the same as at one level above
! Temperature is taken as 2 m temperature         
!**************************************************************************

      uuh(:,:,1,n)=u10(:,:,1,n)
      vvh(:,:,1,n)=v10(:,:,1,n)
      if(TTLactiv) qvh(:,:,1,n)=qvh(:,:,2,n)
      tth(:,:,1,n)=tt2(:,:,1,n)

      if(iumax/=nuvz-1) stop 'READWIND: NUVZ NOT CONSISTENT'
      if(z_motion.and.(iwmax/=nwz))    stop 'READWIND: NWZ NOT CONSISTENT'

      deallocate (zsec4,inbuff,isec2,zsec2)
!     deallocate (ewss,nsss)

      return    
888   write(*,*) ' #### FLEXPART MODEL ERROR! WINDFIELD         #### ' 
      write(*,*) ' #### ',wfname(indj),'                    #### '
      write(*,*) ' #### IS NOT GRIB FORMAT !!!                  #### '
      stop 'Execution terminated'
999   write(*,*) ' #### FLEXPART MODEL ERROR! WINDFIELD         #### ' 
      write(*,*) ' #### ',wfname(indj),'                    #### '
      write(*,*) ' #### CANNOT BE OPENED !!!                    #### '
      stop 'Execution terminated'

      end
!
!=====|==1=========2=========3=========4=========5=========6=========7==
!
!$Log: 
