!$Id: 
!
!=======================================================================
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ FLEXPART @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!=====|==1=========2=========3=========4=========5=========6=========7==
!
      function satratio(press,temp)
!                         i      i
!*******************************************************************************
!                                                                              *
!     Calculates the saturation mixing ratio of water vapor over ice
!     Using formula provided in K. Emanuel's book
!     Calibrated for temperatures in the TTL
!     Result is in mass mixing ratio (kg/kg)
!                                                                              *
!     Author: B. Legras
!                                                                              *
!     31 Jan 2004
!     23 Aug 2007: correction: 
!                  press assumed to be in Pa is divided by 100
!                  to fit in the formula in hPa
!                                                                              *
!*******************************************************************************
!                                                                              *
! Variables:     
!
!
!*******************************************************************************
 
 implicit none
 real satratio
 real, intent(in):: press, temp
 real estar, satppmv
 ! saturation pressure
 estar=1.0008*exp(23.33086-(6111.72784/(temp))+0.15215*alog(temp))
 ! saturation in volume mixing ratio (ppmv)
 satppmv=1e6*estar/(0.01*press-estar)
 ! saturation in mass mixing ratio (kg/kg)
 satratio=0.622e-6 * satppmv    
 return
 end
!
!=====|==1=========2=========3=========4=========5=========6=========7==
!
!$Log: 
