!$Id:
!
!=======================================================================
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ FLEXPART @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!=====|==1=========2=========3=========4=========5=========6=========7==
!
      subroutine verttransformB(n)
!                              i  i   i   i
!-cv      subroutine verttransformB(n,uuh,vvh,tth,qvh,wwh)
!*******************************************************************************
!                                                                              *
!     This subroutine calculates the pressure vertical velocity on             *
!     the eta grid by adding the correction due to horizontal advection        *
!     and dPs/dt to the eta vertical velocity.                                 *
!     It also calculates the velocities in the polar region in stereographic   *
!     map.                                                                     *
!     It will calculate the density and its vertical derivative when it is     *
!     necessary in the future.                                                 *
!                                                                              *
!     Author: B. Legras                                                        *
!     date:   6 April 2002                                                     *
!                                                                              *
!     from a previous version of verttransform.f                               *
!
!     Modifications
!     (...)
!     5/12/07 Calculation of the velocity at the pole for southern
!     hemisphere. Previous calculation (stiil in the official version
!     was much too complicated, though correct, and was inducing
!     occasonnaly spurious floating point exceptions
!                                                                              *
! Variables:                                                                   *
! nx,ny,nz                        field dimensions in x,y and z direction      *
! uuh(0:nxmax,0:nymax,nuvz,2)     wind components in x-direction [m/s]         *
! vvh(0:nxmax,0:nymax,nuvz,2)     wind components in y-direction [m/s]         *
! wwh(0:nxmax,0:nymax,nwz,2)      wind components in z-direction [deltaeta/s]  *
! tth(0:nxmax,0:nymax,nuvz,2)     temperature [K]                              *
! ps(0:nxmax,0:nymax,2)           surface pressure [Pa]                        *
!                                                                              *
!*******************************************************************************

      use commons
      implicit none

      integer ix,jy,iz,n,ix1,jy1,ixp,jyp,ixi,nold
      real pint,pb,pt,pih,ut,vt,ub,vb
      real xlon,ylat,xlonr
      real uupolaux,vvpolaux,wdummy
      real dPsdx,dPsdy,dPsdt,uint,vint

      parameter(pih=pi/180.)

      logical init
      save init
      data init/.true./

! During the first call, the estimate of dPs/dt is done with the next time
! and the maximum height of the model is set (assuming constant pressure 
! level at the top)

      if(init) then
        nold=2 ; init=.false. ; zmax=-log(akm(nwz)/p0) 
      else
        nold=3-n 
      endif

!      print *,'verttran> ',uuh(186,136,24,n),vvh(186,136,24,n),wwh(186,136,24,n)

! Loop over the whole grid
!*************************

!      do jy=0,ny-1
!        do ix=0,nx-1

!       Nothing to be done for the horizontal wind

!       Calculation of rho and drhodz to be done here when necessary
!        enddo
!      enddo

!****************************************************************
! Compute slope of eta levels in windward direction and resulting
! vertical wind correction
!****************************************************************
      if( z_motion.and.correct_vertwind ) then
      do jy=1,ny-2
        if(xglobal) then ; ixi=0 ; else ; ixi=1 ; endif
        do ix=ixi,nx-2
!         velocities in grid units per s 
          ub=uuh(ix,jy,1,n)*dxconst/cos((float(jy)*dy+ylat0)*pih)
          vb=vvh(ix,jy,1,n)*dyconst
          pb=akz(1)+bkz(1)*ps(ix,jy,1,n)
          do iz=1,nwz-1
!           Wind and pressure on adjacent levels
            ut=uuh(ix,jy,iz+1,n)*dxconst/cos((float(jy)*dy+ylat0)*pih)
            vt=vvh(ix,jy,iz+1,n)*dyconst
            pt=akz(iz+1)+bkz(iz+1)*ps(ix,jy,1,n)
            pint=akm(iz)+bkm(iz)*ps(ix,jy,1,n)
!----------- temporary test
!            if((pint.lt.pt).or.(pint.gt.pb)) then
!              print *,'verttransform> ALARM!!: bad vertical ordering'
!              print *,ix,jy,iz,ps(ix,jy,1,n)
!              print *,pb,pint,pt
!              print *,akz(iz+1),akm(iz),akz(iz)
!              print *,bkz(iz+1),bkm(iz),bkz(iz)
!              print *,wwh(ix,jy,iz,n)
!            endif
!-----------
!           Interpolation of horizontal wind  
            uint=(log(pint/pt)*ub + log(pb/pint)*ut)/log(pb/pt)
            vint=(log(pint/pt)*vb + log(pb/pint)*vt)/log(pb/pt)
!           Horizontal derivative of Ps
            ix1=modulo(ix-1,nx)          
!----------- temporary test
!            if((ix1.lt.0).or.(ix1.gt.nx-1)) then
!              print *,'verttransform> ALARM!!: ix1 <0'
!            endif
!-----------          
            jy1=jy-1
            ixp=ix+1
            jyp=jy+1
!           horizontal Ps gradient in Pa per grid unit
            dPsdx = (Ps(ixp,jy,1,n)-Ps(ix1,jy,1,n))/2.
            dPsdy = (Ps(ix,jyp,1,n)-Ps(ix,jy1,1,n))/2.

!           Estimation of temporal derivative of Ps at constant eta
!           unit Pa per s
            dPsdt = (Ps(ix,jy,1,n)-Ps(ix,jy,1,nold)) / &
                    (memtime(2) - memtime(1))

! TEST debut
!            if((ix==186).and.(jy==136).and.(iz==24)) then
!              print *,uint,vint,bkm(iz)
!              print *,dPsdx,dPsdy,dPsdt
!              print *,memtime(2),memtime(1)
!            endif
! TEST fin

!           Correction of vertical velocity
!           by horizontal advection and  dPs/dt
            wwh(ix,jy,iz,n) = wwh(ix,jy,iz,n)          &
              + (dPsdx*uint + dPsdy*vint + dPsdt) *bkm(iz)

            ub=ut ; vb=vt ; pb=pt
          enddo
        enddo
      enddo
      endif
!***  For the moment, the vertical velocities are not corrected
!***  on the first and last latitude and on the first and last 
!***  longitude when xglobal=.false.
!***  TODO: apply first order estimate of spatial derivatives 
!***  on the edges


! If north pole is in the domain, calculate wind velocities in polar
! stereographic coordinates
!*******************************************************************
 
      if (nglobal) then
        do jy=int(switchnorthg)-2,ny-1
          ylat=ylat0+float(jy)*dy
          do ix=0,nx-1
            xlon=xlon0+float(ix)*dx
            do iz=1,nuvz
              call cc2gll(northpolemap,ylat,xlon,uuh(ix,jy,iz,n), &
                vvh(ix,jy,iz,n),uupol(ix,jy,iz,n),                &
                vvpol(ix,jy,iz,n))
            enddo
          enddo
        enddo
         
! CALCULATE FFPOL, DDPOL FOR CENTRAL GRID POINT
! modified because the original version, though correct, was totally crazy
        do iz=1,nuvz
          xlon=xlon0+float(nx/2-1)*dx
          xlonr=xlon*pi/180.
          jy=ny-1
          call cc2gll(northpolemap,90.,xlon,             &
            uuh(nx/2-1,jy,iz,n),vvh(nx/2-1,ny-1,iz,n), &
            uupolaux,vvpolaux)
          do ix=0,nx-1
            uupol(ix,jy,iz,n)=uupolaux
            vvpol(ix,jy,iz,n)=vvpolaux 
          enddo
        enddo
  
! Fix: Set W at pole to the zonally averaged W of the next equator-
! ward parallel of latitude
 
        if(z_motion) then
          do iz=1,nwz
            wdummy=0.
            do ix=0,nx-1
              wdummy=wdummy+wwh(ix,ny-2,iz,n)
            enddo
            wdummy=wdummy/float(nx)
            do ix=0,nx-1
              wwh(ix,ny-1,iz,n)=wdummy
            enddo
          enddo
        endif
      endif 

 
! If south pole is in the domain, calculate wind velocities in polar
! stereographic coordinates
!*******************************************************************
 
      if (sglobal) then
        do jy=0,int(switchsouthg)+3
          ylat=ylat0+float(jy)*dy
          do ix=0,nx-1
            xlon=xlon0+float(ix)*dx
            do iz=1,nuvz
              call cc2gll(southpolemap,ylat,xlon,uuh(ix,jy,iz,n), &
                vvh(ix,jy,iz,n),uupol(ix,jy,iz,n),vvpol(ix,jy,iz,n))
            enddo
          enddo
        enddo
      
        do iz=1,nuvz
! CALCULATE FFPOL, DDPOL FOR CENTRAL GRID POINT
          xlon=xlon0+float(nx/2-1)*dx
          xlonr=xlon*pi/180.
          jy=0
          call cc2gll(southpolemap,-90.,xlon,      &
            uuh(nx/2-1,jy,iz,n),vvh(nx/2-1,0,iz,n), &
            uupolaux, vvpolaux)
          do ix=0,nx-1
            uupol(ix,jy,iz,n)=uupolaux
            vvpol(ix,jy,iz,n)=vvpolaux
          enddo
        enddo
  
! Fix: Set W at pole to the zonally averaged W of the next equator-
! ward parallel of latitude
 
        if(z_motion) then
          do iz=1,nwz
            wdummy=0.
            do ix=0,nx-1
              wdummy=wdummy+wwh(ix,1,iz,n)
            enddo
            wdummy=wdummy/float(nx)
            do ix=0,nx-1
              wwh(ix,0,iz,n)=wdummy
            enddo
          enddo
        endif
      endif

!     print *,'verttran> ',uuh(186,136,24,n),vvh(186,136,24,n),wwh(186,136,24,n)

      return
      end
!
!=====|==1=========2=========3=========4=========5=========6=========7==
!
!$Log:
