!$Id: calcpar.f,v 1.1.1.1 2001/10/03 17:53:13 legras Exp $
!
!=======================================================================
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ FLEXPART @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!=====|==1=========2=========3=========4=========5=========6=========7==
!
      subroutine calcpar(n)
C-cv      subroutine calcpar(n,tth,qvh,uuh,vvh)
C                        i  i   i
********************************************************************************
*                                                                              *
*     Computation of several boundary layer parameters needed for the          *
*     dispersion calculation and calculation of dry deposition velocities.     *
*     All parameters are calculated over the entire grid.                      *
*                                                                              *
*     Author: A. Stohl                                                         *
*                                                                              *
*     21 May 1995                                                              *
*                                                                              *
* ------------------------------------------------------------------           *
*     Petra Seibert, Feb 2000:                                                 *
*     convection scheme:                                                       *
*     new variables in call to richardson                                      *
*                                                                              *
********************************************************************************
*  Changes, Bernd C. Krueger, Feb. 2001:
*        Variables tth and qvh (on eta coordinates) in common block
!           B. Legras, Apr 2002
!        Two times for uuh and vvh, in common
********************************************************************************
*                                                                              *
* Variables:                                                                   *
* n                  temporal index for meteorological fields (1 to 3)         *
*                                                                              *
* Constants:                                                                   *
*                                                                              *
*                                                                              *
* Functions:                                                                   *
* scalev             computation of ustar                                      *
* obukhov            computatio of Obukhov length                              *
*                                                                              *
********************************************************************************

      include 'includepar'
      include 'includecom'

      integer n,ix,jy,i
      real ttlev(nuvzmax),qvlev(nuvzmax),obukhov,scalev,ol,hmixplus
      real ulev(nuvzmax),vlev(nuvzmax),ew,rh,vd(maxspec),subsceff

C Loop over entire grid
***********************

      do 10 jy=0,ny-1
        do 10 ix=0,nx-1

C 1) Calculation of friction velocity
*************************************

          ustar(ix,jy,1,n)=scalev(ps(ix,jy,1,n),tt2(ix,jy,1,n),
     +    td2(ix,jy,1,n),surfstr(ix,jy,1,n))
          if (ustar(ix,jy,1,n).le.1.e-8) ustar(ix,jy,1,n)=1.e-8

C 2) Calculation of inverse Obukhov length scale
************************************************

          ol=obukhov(ps(ix,jy,1,n),tt2(ix,jy,1,n),td2(ix,jy,1,n),
     +    tth(ix,jy,2,n),ustar(ix,jy,1,n),sshf(ix,jy,1,n),akm,bkm)
          if (ol.ne.0.) then
            oli(ix,jy,1,n)=1./ol
          else
            oli(ix,jy,1,n)=99999.
          endif


C 3) Calculation of convective velocity scale and mixing height
***************************************************************

          do 20 i=1,nuvz
            ulev(i)=uuh(ix,jy,i,n)
            vlev(i)=vvh(ix,jy,i,n)
            ttlev(i)=tth(ix,jy,i,n)
20          qvlev(i)=qvh(ix,jy,i,n)

          call richardson(ps(ix,jy,1,n),ustar(ix,jy,1,n),ttlev,qvlev,
     +    ulev,vlev,nuvz,akz,bkz,sshf(ix,jy,1,n),tt2(ix,jy,1,n),
     +    td2(ix,jy,1,n),hmix(ix,jy,1,n),wstar(ix,jy,1,n),hmixplus)

          if(lsubgrid.eq.1) then
            subsceff=min(excessoro(ix,jy),hmixplus)
          else
            subsceff=0
          endif
*
* CALCULATE HMIX EXCESS ACCORDING TO SUBGRIDSCALE VARIABILITY AND STABILITY
*
          hmix(ix,jy,1,n)=hmix(ix,jy,1,n)+subsceff
          hmix(ix,jy,1,n)=max(hmixmin,hmix(ix,jy,1,n)) ! minim PBL height 150 m
          hmix(ix,jy,1,n)=min(hmixmax,hmix(ix,jy,1,n)) ! maxim PBL height 3500 m

C 4) Calculation of dry deposition velocities
*********************************************

          if (DRYDEP) then
            z0(4)=0.016*ustar(ix,jy,1,n)*ustar(ix,jy,1,n)/ga
            z0(9)=0.016*ustar(ix,jy,1,n)*ustar(ix,jy,1,n)/ga

C Calculate relative humidity at surface
****************************************
!            rh=ew(td2(ix,jy,1,n))/ew(tt2(ix,jy,1,n))
!
!            call getvdep(n,ix,jy,ustar(ix,jy,1,n),
!     +      tt2(ix,jy,1,n),ps(ix,jy,1,n),1./oli(ix,jy,1,n),
!     +      ssr(ix,jy,1,n),rh,lsprec(ix,jy,1,n)+convprec(ix,jy,1,n),vd)
!
!            do 30 i=1,nspec
!30            vdep(ix,jy,i,n)=vd(i)

          endif

10        continue


      return
      end

!
!=====|==1=========2=========3=========4=========5=========6=========7==
!
!$Log: calcpar.f,v $
!Revision 1.1.1.1  2001/10/03 17:53:13  legras
!FLEXPART 4.0 - original version
!
!
