subroutine fix_perturb


!********************************************************************************
!   	 Fixes perturbations used in the calculation of lyapunov charateristic 	*
! 	 exponents								*
!									      	*
!	 Ignacio PISSO April 2004				      	       	*
!********************************************************************************
!                                                                              	*
! Variables:                                                                   	*
!                                                                              	*
!********************************************************************************

 use commons
 implicit none
 
integer i,j, j1 

real, dimension(7,4) ::  p , b
real, dimension(3)   :: v,vx,vy,v1,v2,v3
real deltaz

!main boucle 
!************************************************************************************* 
j1 = numpart_np 
do i=1,numpart_np
 	                          
   do j=1,7  
        call grid2cart(p(j,1),p(j,2),p(j,3),xtra1(i),ytra1(i))
        p(j,4) = ztra1(i)
	call sphere(p(j,:))			
   enddo

	    
! Initialisation 2D base on the surface of the sphere 
!*********************************************************************************

    v  = p(1,1:3)
    vx = (/ 1, 0, 0/)
    vy = (/ 0, 1, 0/)
  
    v1 =  vx - v*prod3(v,vx)
    v1 =  v1/norm3(v1)
    v2 =  vy - v*prod3(v,vy) - v1*prod3(vy,v1) 
    v2 =  v2/norm3(v2)
   
  				
!Perturbations
!*********************************************************************************
		
    v1 = v1*(delta_hor/r_earth)
    v2 = v2*(delta_hor/r_earth)	
    deltaz = delta_ver/height_scale
								
    b        = 0.
    b(2,1:3) = v1					
    b(3,1:3) = v2	
    b(4,4)   = deltaz
    b(5,1:3) = -v1					
    b(6,1:3) = -v2
    b(7,4)   = -deltaz
    
    p = p + b
    
    do j=1,6
      call sphere(p(j+1,:))
      call cart2grid(xtra1(j1+j),ytra1(j1+j),p(j+1,1),p(j+1,2),p(j+1,3))
      ztra1(j1+j) = p(j+1,4)
      itra1(j1+j) = itra1(i)
      itramem(j1+j) = itra1(i)
    enddo
    j1=j1+6

enddo    !end main boucle

print * , "Perturbations initialized " 			

return
 
 
!*********************************************************************************** 
 
contains

!***********************************************************************************
!Scalar product in 3 D, within the unit sphere world
!***********************************************************************************   
 
 real function prod3(u,v)
 real, dimension(3) :: u
 real, dimension(3) :: v
 prod3 = u(1)*v(1)+u(2)*v(2)+u(3)*v(3)
 return
 end function prod3
 
!************************************************************************************
!Norm in 3 D, within the unit sphere world
!************************************************************************************
 real function norm3(v)
 real, dimension(3) :: v
 norm3 =  sqrt((v(1))**2 + (v(2))**2 + (v(3))**2 ) 
 return
 end function norm3
 
!***************************************************************************************************************************************************************************
!Normalisation 4D (normal)
!***************************************************************************************************************************************************************************  
 
 subroutine sphere(v)
 real, dimension(4) :: v
 real arrow
 arrow=1/sqrt(v(1)**2 + v(2)**2 + v(3)**2)
 v(1) = v(1)*arrow
 v(2) = v(2)*arrow
 v(3) = v(3)*arrow
 end subroutine sphere
  
 end subroutine fix_perturb
