subroutine gramlam(hh,b,lambdagram) 

!*****************************************************************************************************
!   	 Orthogonalize perturbations used in the calculation of Lyapunov characteristic exponents
!	 					             		  			     
!	 Ignacio PISSO April 2004                                              			
! 	 						                       		
!*****************************************************************************************************
!                                                                              			
! Variables:                                                                   		
!                                                                              	
!*****************************************************************************************************

use commons
implicit none
 
real, dimension(3,4) , intent (in) 	:: hh 

integer 				   i,j,j1,k,m
real					:: area1, vol
real, dimension(4)   			:: h3p , h2p 

real, dimension(3,4) , intent (out)	:: b
real, dimension(3)   , intent (out)	:: lambdagram

!*****************************************************************************************************

b(1,:) = hh(1,:)/norm4(hh(1,:))
lambdagram(1) = log(norm4(hh(1,:)))                    
h2p = hh(2,:) - prod4(b(1,:),hh(2,:))*b(1,:) 
b(2,:) = h2p/norm4(h2p)
area1 = cross4(hh(1,:),hh(2,:))
lambdagram(2)= log(area1) - lambdagram(1)            
h3p =  hh(3,:) - prod4(b(1,:),hh(3,:))*b(1,:) - prod4(b(2,:),hh(3,:))*b(2,:)
b(3,:) = h3p/norm4(h3p)
vol = abs(area1*norm4(h3p))
lambdagram(3) =  log(vol) - lambdagram(2) - lambdagram(1)

!test-----
!write(*,'(a,3g12.5)') ' gramlam> ',lambdagram(1),lambdagram(2),lambdagram(3)
!write(*,'(a,3g12.5)') ' gramlam> ',norm4(b(1,:)),norm4(b(2,:)),norm4(b(3,:))
!write(*,'(a,3g12.5)') ' gramlam> ',prod4(b(1,:),b(2,:)),prod4(b(2,:),b(3,:)),prod4(b(3,:),b(1,:))
write(*,'(a,3g12.5)') ' gramlam> ', &
   180/pi * asin(prod4(hh(1,:),hh(2,:))/(norm4(hh(1,:))*norm4(hh(2,:)))), &
   180/pi * asin(prod4(hh(2,:),hh(3,:))/(norm4(hh(2,:))*norm4(hh(3,:)))), &
   180/pi * asin(prod4(hh(3,:),hh(1,:))/(norm4(hh(3,:))*norm4(hh(1,:))))
!---------
   
contains
 
!*****************************************************************************************************    
!Scalar product in 4 D
!*****************************************************************************************************
 
 real function prod4(u,v)
 real, dimension(4) :: u
 real, dimension(4) :: v
 prod4 = ((r_earth/delta_hor)**2)*(u(1)*v(1)+u(2)*v(2)+u(3)*v(3)) &
        +((height_scale/delta_ver)**2)*u(4)*v(4)
 return
 end function prod4
  
!*****************************************************************************************************
!Norm in 4 D
!*****************************************************************************************************
 
 real function norm4(v)
 real, dimension(4) :: v
 norm4 =  sqrt(((r_earth/delta_hor)**2)*(v(1)**2 + v(2)**2 + v(3)**2) &
             + ((height_scale/delta_ver)*v(4))**2) 
 return
 end function norm4

!*****************************************************************************************************
!Normalisation 4D (normal)
!*****************************************************************************************************  
 
 subroutine sphere(v)
 real, dimension(4) :: v
 real arrow
 arrow=1/sqrt(v(1)**2 + v(2)**2 + v(3)**2)
 v(1) = v(1)*arrow
 v(2) = v(2)*arrow
 v(3) = v(3)*arrow
 end subroutine sphere
 
!*****************************************************************************************************
!Cross product
!*****************************************************************************************************  
   
 real function cross4(v,w)
 real, dimension(4) :: v,w
 real prod, norm_v2, norm_w2
 norm_v2 = ((r_earth/delta_hor)**2)*(v(1)**2 + v(2)**2 + v(3)**2) &
         + ((height_scale/delta_ver)*v(4))**2
 norm_w2 = ((r_earth/delta_hor)**2)*(w(1)**2 + w(2)**2 + w(3)**2) &
         + ((height_scale/delta_ver)*w(4))**2
 prod = ((r_earth/delta_hor)**2)*(w(1)*v(1)+w(2)*v(2)+w(3)*v(3)) &
      + ((height_scale/delta_ver)**2)*w(4)*v(4)
 cross4 = sqrt(norm_v2 * norm_w2 - prod**2)
 return
 end function cross4
  
end subroutine gramlam
